/*jslint node: true */
/*global define */

define(['angular', 'angularSnap', 'accessibleTimePickerServices', 'TimeControls'], function (angular) {
    'use strict';

    var app = angular.module('safe-patient-handling', [
        'ngSanitize',
        'ngTouch',
        'ui.router',
        'ui.bootstrap',
        'ui.utils',
        'snap',
        'unsavedChanges',
        'delayManager',
        'focusManager',
        'accessibleDatePickerServices',
        'accessibleTimePickerServices',
        'DateControls',
        'TimeControls',
        'AccessibleAccordion'
    ]);

    app.config(function ($controllerProvider, $provide, $httpProvider, snapRemoteProvider) {
        $httpProvider.defaults.useXDomain = true;
        delete $httpProvider.defaults.headers.common['X-Requested-With'];
        $httpProvider.interceptors.push('busyHttpInterceptor');

        snapRemoteProvider.globalOptions.touchToDrag = false;
    });

    app.directive('scrollContainer', function ($window, $timeout) {
        return function (scope, element) {
            angular.element($window).bind("resize", function () {
                $timeout(function () {
                    element.height("calc(100% - 79px)");
                });
            });
            scope.$watch(function () {
                return $window.innerHeight + $window.innerWidth;
            }, function () {
                angular.element($window).trigger("resize");
            })
        };
    });

    app.directive('showRecommendation', function ($compile, $parse, $sce) {
        return {
            restrict: 'A',
            scope: {
                bariatric: '@',
                bariatricFlag: '@',
                showRecommendation: '@'
            },
            compile: function (element, attrs) {
                return function (scope, element, attrs) {
                    var template,
                        bariatric,
                        notes;

                    if (attrs.bariatric == 'true') {
                        scope.bariatric = true;
                    } else {
                        scope.bariatric = false;
                    }

                    // ugly, but there's no way around it in angular at the moment
                    // see Angular author/contributor Misko's answer at
                    // http://stackoverflow.com/a/10646761
                    // for an alternate example (though not repeatable as this is)
                    template = [ attrs.showRecommendation ].join('');

                    element.replaceWith($compile(template)(scope));
                };
            }
        };
    });

    return app;
});
