/*jslint node: true */
/*global define */


define(['app', 'global', 'ClearController'], function (app, global, ClearController) {
    "use strict";
    app.directive('menuInclude', function () {
        return {
            replace: true,
            restrict: 'A',
            templateUrl: "src/container/components/menu/menu_template.html",
            controller: function ($scope, $rootScope, $q, $http, $window, $timeout, $modal, $state, $filter, pageService, snapRemote, focusService) {

                var closeFocusHelper;

                snapRemote.getSnapper().then(function (snapper) {
                    snapper.on('animated', function () {
                        $timeout(function () {
                            switch (snapper.state().state) {
                                case "left":
                                    focusService.focusElement(angular.element('.snap-drawer-left .iOS-VO-overlap-accessible.panel-title'), true);
                                    break;
                                case "right":
                                    focusService.focusElement(angular.element('.snap-drawer-right .iOS-VO-overlap-accessible.panel-title'), true);
                                    angular.element($window).trigger('resize');
                                    break;
                            }
                        });
                    });
                    snapper.on('open', function () {
                        angular.element('.snap-content').attr('aria-hidden', 'true');
                        closeFocusHelper = "main";
                    });
                    snapper.on('close', function () {
                        angular.element('.snap-content').attr('aria-hidden', 'false');
                        if (closeFocusHelper === "main") {
                            focusService.focusMain();
                        }
                    });
                    $scope.snapper = snapper;
                });

                var statesList = $state.get();
                $scope.navItems = $filter('orderBy')(statesList.filter(function (val) {
                    return val.appNavIndex > 0;
                }), "appNavIndex").map(function (val) {
                    val.data.id = val.data.appletName.replace(/\s/g, "");
                    return val;
                });

                $http.get('src/container/components/menu/user-menu-items.json')
                    .then(function (menuItems) {
                        $scope.userItems = menuItems.data.menuItems;

                        $timeout(function () {
                            angular.element($window).trigger('resize');
                        });
                    });

                angular.element($window).resize(function () {
                    resizeList(angular.element('div[snap-drawer=right]'));
                    resizeList(angular.element('div[snap-drawer=left]'));
                });

                // var isAuthenticated = authenticationService.isAuthenticated();
                $scope.authenticationFilter = function (menuItem) {
                    // if ($state.current.module === "preventNavigation" && $scope.navItems.indexOf(menuItem) !== -1) {
                    // 	return false;
                    // }
                    // if (menuItem.data && menuItem.data.requiresAuth === true|| menuItem.requiresAuth === true) {
                    // 	return isAuthenticated;
                    // }
                    // if (menuItem.data && menuItem.data.requiresAuth === false || menuItem.requiresAuth === false) {
                    // 	return !isAuthenticated;
                    // }

                    return true;
                };

                $scope.openClearAssessmentModal = function () {
                    var modalInstance = $modal.open({
                            windowTemplateUrl: 'src/ui-components/modals/helper/modal-window_template.html',
                            templateUrl: 'src/ui-components/modals/clearData/clear_data_template.html',
                            controller: 'ClearController'
                        });

                    modalInstance.result.finally(function () {
                        focusService.focusElement(angular.element('h3#modal-header'), true);
                    });
                };

                function resizeList(listContainer) {
                    var windowHeight;

                    windowHeight = window.innerHeight;

                    var totalListHeight = 0;

                    totalListHeight += listContainer.find('.panel-header').outerHeight();
                    listContainer.find('ul').each(function (index, element) {
                        totalListHeight += angular.element(element).outerHeight(false);
                    });

                    var margin = windowHeight - totalListHeight;
                    if (margin < 0) {
                        margin = 0;
                    }

                    listContainer.find('ul').first().css('margin-bottom', margin + 'px');
                }

                $scope.closeMenu = function () {
                    angular.element('.snap-content').attr('aria-hidden', 'false');
                    closeFocusHelper = "main";
                };

                $scope.openLink = function (link) {
                    switch (link) {
                        // case "logout":
                        // 	authenticationService.logoutRedirectToLaunchpad();
                        // 	break;
                        case "main.resources":
                            pageService.redirectToResources();
                            break;
                        case "launchpad":
                            pageService.redirectToLaunchpad();
                            break;
                    }
                };

                $scope.openModal = function (item) {
                    var modalInfo = item.modal,
                        modalInstance = $modal.open({
                            windowTemplateUrl: 'src/ui-components/modals/helper/modal-window_template.html',
                            templateUrl: modalInfo.templateUrl,
                            controller: modalInfo.controller
                        });

                    modalInstance.result.finally(function () {
                        focusService.focusElement(angular.element('button[id=' + item.id + ']'));
                    });
                };

                $scope.openSubApp = function (state, params) {
                    $state.go(state.name, params);
                    closeFocusHelper = "section";
                };

                $scope.isSelected = function (item) {
                    return $state.includes(item.name);
                };
            }
        };
    });
});