define(['angular', 'app'], function (angular, app) {
	"use strict";
	app.service('resourceDirectory', function ($http, $q) {
		var resourceDirectory = {},
			helper = {},
			isReady;

		helper.getDirectoryPath = function () {
			var deferred = $q.defer();

			if (window.cordova) {
				pList.fetch(
					function (pList) {
						deferred.resolve(pList["MHPResourceDirectoryURL"]);
					},
					function (error) {
						deferred.reject(error);
					});
			} else {
				var location = window.location,
                    path = location.origin || (location.protocol + "//" + location.hostname + (location.port ? ':' + location.port: '')),
					servicesPath = '/MobileHealthPlatformWeb/rest/public/resource-directory/json';

				path = path + servicesPath;
				deferred.resolve(path);
			}

			return deferred.promise;
		};

		resourceDirectory.fetch = function () {

			if (isReady) {
				return isReady.promise;
			}

			isReady = $q.defer();

			helper.getDirectoryPath().then(function (path) {
				$http.get(path).success(function (response) {
					var directory = {};
					response['link'].forEach(function (link) {
						directory[link['title']] = link['href'];
					});
					isReady.resolve(directory);
				}).error(function (error) {
					isReady.reject(error);
					isReady = null;
				});
			}, function (error) {
				isReady.reject(error);
			});

			return isReady.promise;

		};

		return resourceDirectory;
	});
});
