/*jslint node: true, unparam: true */
/*global define, controller */

define(['angular', 'app'], function (angular, app) {
    'use strict';

    app.controller('CPAAPrimaryWrapperController', [
        '$scope', '$state', '$stateParams', '$filter', '$modal', '$log', 'modalService', 'focusService', 'AssessmentInstanceBuilder', 'RecommendationEngine',
        function ($scope, $state, $stateParams, $filter, $modal, $log, modalService, focusService, AssessmentInstanceBuilder, RecommendationEngine) {
            // Controller is used to to wrap the app..
            // Can be first if single-task, or second if multi-task
            // Flow:
            //      - On all state changes we set the section header.
            //      - Also will clear the assessment and open the info button.
            $log.info('CPAAPrimaryWrapperController');

            // ---------------------------------------------------------------------------------------------------------
            // -    Module Functions
            // ---------------------------------------------------------------------------------------------------------
            $scope.clearAssessment = function () {
                var modalContent = {
                    actionButtonText: 'OK',
                    closeButtonText: 'Cancel',
                    headerText: 'Confirm clear data',
                    bodyText: 'Confirming this action will clear all assessment question responses, selected tasks, and recommendations.',
                };

                modalService.showModal({}, modalContent).then(
                    //  What to do when OK button is tapped
                    function () {
                        var service = AssessmentInstanceBuilder.getInstance();
                        service.delete();
                        RecommendationEngine.clearSelectedAlgorithms();
                        // If we don't clear chosen, the previous Algorithm will be pre-selected.
                        RecommendationEngine.clearChosenAlgorithms();
                        $state.go('main.two-panel.secondary-navigation.cpaa');
                    },
                    //  What to do when Cancel button is tapped
                    function () {
                        // When modal is closed refocus on the button that opened it.
                        focusService.focusElement(angular.element('button[title="Clear Assessment"]'));
                    }
                );
            };

            $scope.infoButton = function () {
                var modalContent = {
                    closeButtonText: 'OK',
                    headerText: 'Safe Patient Handling and Mobility Patient Assessment/Algorithms/Care Plan',
                    bodyContentUrl: 'src/ui-components/modals/info/info_template.html',
                };

                modalService.showModal({}, modalContent).then(function () {
                }, function () {
                    focusService.focusElement(angular.element('button[title="Info"]'));
                });
            };

            // ---------------------------------------------------------------------------------------------------------
            // -    Change Events
            // ---------------------------------------------------------------------------------------------------------
            $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams, $stateParams) {
                $stateParams = toState;

                $log.info('$stateChangeSuccess');
                $log.info($stateParams);
                //  $log.info(toState);
                //  $log.info(toParams);
                //  So sometimes we have $stateParams coming in; if so we want to keep them.
                //  We're also looking specifically at the single vs. multi task; so we can assign the correct sectionName.
                //  $scope.sectionName = toState.data.primaryHeader || $filter('filter')(toState.data.paramSections, {paramValue: $stateParams.section}, true)[0].sectionName;
                $stateParams.section = toParams.section;
                if ($stateParams.section === 'single-task') {
                    $scope.sectionName = toState.data.primaryHeader || toState.data.paramSections[1].sectionName;
                    $stateParams.nowDoAssessment = toParams.nowDoAssessment || toState.data.paramSections[1].nowDoAssessment;
                } else if ($stateParams.section === 'multi-tasks') {
                    $scope.sectionName = toState.data.primaryHeader || toState.data.paramSections[0].sectionName;
                    $stateParams.nowDoAssessment = toParams.nowDoAssessment || toState.data.paramSections[0].nowDoAssessment;
                } else {
                    $scope.sectionName = '';
                }
            });
        }
    ]);
});