/*jslint node: true */
/*global define */

define(['angular', 'app'], function (angular, app) {
    "use strict";
    app.filter('myFilter', function () {
        return function (inputs, filterValues) {
            var output = [];
            angular.forEach(inputs, function (input) {
                if (filterValues.indexOf(input.value) !== -1)
                    output.push(input);
            });
            return output;
        };
    });

    app.service('AssessmentService', function ($window, Constants, $log, $state, $stateParams, $filter) {
        // Service is used to track the Questions.
        // Functions
        //      - getAnswerSheet(assessment) - returns an object string of the questions and answers.
        //      - returnAllQuestions
        //      - returnAllQuestionsWithAnswers -
        //      - returnSpecificAlgorithmQuestions
        //      - save
        //      - delete
        // Flow:

        $log.info('AssessmentService start');

        // -------------------------------------------------------------------------------------------------------------
        // -    Init Section, setting default values.
        // -------------------------------------------------------------------------------------------------------------
        var Assessment = function () {
            this.assessment = angular.fromJson($window.localStorage.assessment) || angular.copy(Constants.questions);
        };


        // -------------------------------------------------------------------------------------------------------------
        // -    Init Section, setting default values.
        // -------------------------------------------------------------------------------------------------------------
        Assessment.prototype.getAnswerSheet = function (assessment) {
            // assessment:  Not passed in from recommendation. odd.
            //              this.assessment is copied from localStorage.assessment (questions) or the Constants.questions
            //  $log.info('AssessmentService: getAnswerSheet():');
            //  $log.info(assessment);

            // Init
            var answerSheet = {},
                getAnswerLetters;

            assessment = assessment || this.assessment;
            //  $log.info('AssessmentService: getAnswerSheet():');
            //  $log.info(assessment);

            // Sub-functions
            getAnswerLetters = function (choice, options) {
                var letters,
                    index;
;

                // if (angular.isArray(choice)) {
                if (angular.isObject(choice)) {
                    letters = [];
                    angular.forEach(choice, function (value, key) {
                        var newKey = Number(key) + 65;
                        if (value === true) {
                            letters.push(key > -1 ? String.fromCharCode(newKey) : null);
                        }
                        //  $log.info('getAnswerLetters - index, letters: ' + key + ', ' + letters);
                    });
                } else {
                    index = _.indexOf(options, choice);
                    letters = index > -1 ? String.fromCharCode(65 + index) : null;
                    // $log.info('getAnswerLetters - index, letters: ' + index + ', ' + letters);
                }

                //  $log.info('letters follow');
                //  $log.info(letters);

                return letters;
            };

            // Main Function Calls

            // Here is the key bit I think.
            // It takes teh answers (assessment or algorithms) and loops through each one.
            // question.id:         ID of the "Algorithm"
            // question.choice:     not in Constants. from localStorage, chosen answer.
            // question.options:    object of answers.
            angular.forEach(assessment, function (question) {
            //  $log.info('getAnswerLetters returned: (question first)');
            //  $log.info(question);
            //  $log.info(getAnswerLetters(question.choice, question.options));
                answerSheet[question.id] = getAnswerLetters(question.choice, question.options);
            //  $log.info(answerSheet[question.id]);
            });
            // Answer Sheet gives a full list of answers.  Should be stored locally??
            // Object {1: "A", 2: "A", 3: "A", 4: "A", 5: "A", 6: "A", 7: "A", 8: "A", 9: "A", 10: "A", 11: "A", 12: null, 13: "A", 14: "A", 15: "A", 16: "A", 17: "A"}
            //  $log.info('answersheet -------:');
            //  $log.info(answerSheet);
            //  $log.info('AssessmentService: getAnswerSheet(): return answerSheet:');
            //  $log.info(answerSheet);
            return answerSheet;
        };

        //

        // Fetch returns all Questions from Constants.
        Assessment.prototype.returnAllQuestions = function () {
            // $log.info('assessment service: fetch();');
            // return angular.copy(Constants.questions);
            return angular.fromJson($window.localStorage.assessment) || angular.copy(Constants.questions);
        };

        // Fetch returns an array with objects of all Questions and Answers
        Assessment.prototype.returnAllQuestionsWithAnswers = function () {
            var questionsToFill = angular.copy(Constants.questions),
                questionsWithAnswers = angular.copy(angular.fromJson($window.localStorage.assessment));

            questionsToFill.forEach(function (value, index) {
                value = questionsWithAnswers[index].hasOwnProperty('choice') ? questionsWithAnswers[index] : value;
                questionsToFill[index] = angular.copy(value);
            });

            return questionsToFill;

        };

        // returnSpecificAlgorithmQuestions returns the selected questions based on the Selected Algorithms LocalStorage
        // This is done only for the single-task item.
        Assessment.prototype.returnSpecificAlgorithmQuestions = function () {
            // $log.info('assessment service: returnSpecificAlgorithmQuestions();');

            // Init
            var atLeastOneQuestionAnswered = angular.fromJson($window.localStorage.assessment) ? true : false,
                arrayOfQuestionsFromJson = angular.fromJson($window.localStorage.assessment),
                answeredFilteredQuestions = [],
            // This is to get the questions just for the selected questions
                selectedAlgorithm = angular.copy(angular.fromJson($window.localStorage.selectedAlgorithms)),
                // allQuestions = this.assessment,
                allQuestions =  angular.fromJson($window.localStorage.assessment) || angular.copy(Constants.questions),
                allAlgorithms = Constants.algorithms,
                filteredQuestions = [];


            // VALUE: this is the Question.ID we need.
            // KEY: This is just the index order of the questions as they've come in. (0, 1, 2, 3, 4...)
            angular.forEach(selectedAlgorithm, function (algorithm) {
                angular.forEach(algorithm.questions, function (value, key) {
                    var thisQuestion = _.find(allQuestions, {
                        id: value
                    });

                    if (value) {
                        filteredQuestions.push(thisQuestion);
                    }
                });
            });

            if (atLeastOneQuestionAnswered) {
                //make a new array of the assessment with answers that has the same questions as filteredQuestions
                angular.forEach(selectedAlgorithm, function (algorithm) {
                    angular.forEach(algorithm.questions, function (value, index) {
                        var filteredQ = _.find(arrayOfQuestionsFromJson, {
                            id: value
                        });
                        answeredFilteredQuestions.push(filteredQ);
                    });
                });

                //copy answer choices from answeredFilteredQuestions to filteredQuestions if they exist
                filteredQuestions.forEach(function (value, index) {
                    value = answeredFilteredQuestions.hasOwnProperty('choice') ? answeredFilteredQuestions[index] : value;
                    filteredQuestions[index] = angular.copy(value);
                });

            }
            return filteredQuestions;

        };

        Assessment.prototype.save = function (assessment) {
            //multi-task assessment is passed in
            if (assessment.length === 17) {
                this.assessment = angular.copy(assessment);
                $window.localStorage.assessment = angular.toJson(assessment);
            } else { //single-task assessment is passed in
                //add choices to this.assessment
                var questions = this.assessment;
                angular.forEach(questions, function (value) {
                    for (var i = 0; i < assessment.length; i++) {
                        if (value.id === assessment[i].id) {
                            value.choice = angular.copy(assessment[i].choice);
                        }
                    }
                });
                $window.localStorage.assessment = angular.toJson(questions);
            }

        };

        Assessment.prototype.delete = function () {
            // $log.info('AssessmentService.delete');
            delete $window.localStorage.assessment;
            this.assessment = angular.copy(Constants.questions);
        };

        return Assessment;
    });
});


