/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define(['AssessmentService'], function () {
    'use strict';

    describe("The AssessmentService", function () {
        var AssessmentService,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            store = {};

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            answers: {
                '1': 'B',
                '2': 'C',
                '3': 'C',
                '4': 'B',
                '5': 'B',
                '6': 'B',
                '7': 'A',
                '8': 'C',
                '9': 'C',
                '10': 'C',
                '11': 'B',
                '12': null,
                '13': 'C',
                '14': 'B',
                '15': 'A',
                '16': 'C',
                '17': 'B'
            },
            assessment: [
                {
                    "id": 1,
                    "question": "Patient's level of cooperation and comprehension",
                    "options": ["Cooperative, may need prompting, able to follow simple commands/instructions", "Uncooperative, combative, or unpredictable/unable to follow simple commands/instructions"],
                    "choice": "Uncooperative, combative, or unpredictable/unable to follow simple commands/instructions"
                }, {
                    "id": 2,
                    "question": "Weight Bearing Capability",
                    "options": ["Fully Weight Bearing", "Non-Weight Bearing", "Partial Weight Bearing"],
                    "choice": "Non-Weight Bearing"
                }, {
                    "id": 3,
                    "question": "Upper Extremity Strength",
                    "options": ["Full Strength in both upper extremities", "No Strength in either upper extremity", "Strength in one upper extremity"],
                    "choice": "Full Strength in both upper extremities"
                }, {
                    "id": 10,
                    "question": "Balance",
                    "options": ["Ability to maintain sitting balance at edge of bed/chair", "Ability to maintain standing balance", "No sitting or standing balance"],
                    "choice": "No sitting or standing balance"
                }, {
                    "id": 11,
                    "question": "Ability to grasp",
                    "options": ["Full grasp with both hands", "No grasping ability with either hand", "Grasp with one hand"],
                    "choice": "Grasp with one hand"
                }
            ],
            questions: [
                {
                    id: 7,
                    question: 'Is the patient greater than 300 lb or 136 kg?',
                    options: ['Yes', 'No']
                },
                {
                    id: 2,
                    question: 'Weight Bearing Capability',
                    options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing']
                },
                {

                    id: 10,
                    question: 'Balance',
                    options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance']
                }
            ],
            answerSheet: [
                {
                    id: 7,
                    question: 'Is the patient greater than 300 lb or 136 kg?',
                    options: ['Yes', 'No'],
                    selection: ['Yes']
                },
                {
                    id: 2,
                    question: 'Weight Bearing Capability',
                    options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing'],
                    selection: ['Non-Weight Bearing']
                },
                {

                    id: 10,
                    question: 'Balance',
                    options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance'],
                    selection: ['Ability to maintain standing balance']
                }
            ],
            algorithms: [{
                id: 2,
                name: 'Lateral Transfer to/from Supine Positions: Bed, Stretcher, Trolley, Procedure Table',
                flowChart: 'path/to/chart/image.png',
                notes: 'yes test common recommendation',
                questions: [1, 4],
                algorithm: [
                    {
                        id: [1],
                        answerCriteria: '^1B$',
                        recommendation: 0
                    }, {
                        id: [1, 4],
                        answerCriteria: '^1A4B$',
                        recommendation: 0
                    }, {
                        id: [1, 4],
                        answerCriteria: '^1A4A$',
                        recommendation: 1
                    }
                ],
                solution: [
                    {
                        recommendation: 'A Recommendation'
                    }, {
                        recommendation: 'B Recommendation'
                    }
                ]
            }, {
                id: 3,
                name: 'Repositioning in Bed',
                flowChart: 'path/to/chart/image.png',
                questions: [1, 5],
                algorithm: [
                    {
                        id: [1],
                        answerCriteria: '^1B$',
                        recommendation: 0
                    }, {
                        id: [1, 5],
                        answerCriteria: '^1A5B$',
                        recommendation: 0
                    }, {
                        id: [1, 5],
                        answerCriteria: '^1A5A$',
                        recommendation: 1
                    }
                ],
                solution: [
                    {
                        recommendation: 'A Recommendation'
                    }, {
                        recommendation: 'B Recommendation'
                    }
                ],
                'notes': '<ul aria-label="SPECIFIC:">'
            }]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');
            // Sets the selectedAlgorithms to a multi-selected item of 2.
            window.localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms));

            module(function ($provide) {
                $provide.constant('Constants', ConstantsMock);
            });


            inject(function (_AssessmentService_, _$window_) {
                AssessmentService = _AssessmentService_;
                $window = _$window_;
            });
            var service = new AssessmentService();
        });

        describe("should test the load and saving", function () {
            window.localStorage.removeItem('assessment');
            it("returnAllQuestions (localstorage or no)", function () {
                var service = new AssessmentService(),
                    questions = service.returnAllQuestions();

                expect(questions).toEqual(ConstantsMock.questions);

                window.localStorage.setItem('assessment', angular.toJson(ConstantsMock.answerSheet));
                questions = service.returnAllQuestions();

                expect(questions).toEqual(ConstantsMock.answerSheet);
            });
        });
    });
});