/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'text!/base/app/src/modules/cpaa/core/constants/cpaaQuestions.json',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_equipment_0.html'
], function (RecommendationEngine,
             incomingCpaaQuestions,
             incomingCpaaAlgorithm_1_notes,
             incomingCpaaAlgorithm_1_recommendation_0,
             incomingCpaaAlgorithm_1_recommendation_1,
             incomingCpaaAlgorithm_1_recommendation_2,
             incomingCpaaAlgorithm_1_equipment_0) {
    'use strict';

    describe("The RecommendationEngine - Testing Algorithm One", function () {
        var scope,
            RecommendationEngine,
            Constants,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            store = {},
            $sce,
            cpaaQuestions                       = JSON.parse(incomingCpaaQuestions),
            cpaaAlgorithm_1_notes               = incomingCpaaAlgorithm_1_notes.toString(),
            cpaaAlgorithm_1_recommendation_0    = incomingCpaaAlgorithm_1_recommendation_0.toString(),
            cpaaAlgorithm_1_recommendation_1    = incomingCpaaAlgorithm_1_recommendation_1.toString(),
            cpaaAlgorithm_1_recommendation_2    = incomingCpaaAlgorithm_1_recommendation_2.toString(),
            cpaaAlgorithm_1_equipment_0         = incomingCpaaAlgorithm_1_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            assessment: {
                '1': 'B',
                '2': 'C',
                '3': 'C',
                '4': 'B',
                '5': 'B',
                '6': 'B',
                '7': 'A',
                '8': 'C',
                '9': 'C',
                '10': 'C',
                '11': 'B',
                '12': null,
                '13': 'C',
                '14': 'B',
                '15': 'A',
                '16': 'C',
                '17': 'B'
            },
            questions: cpaaQuestions,
            algorithms: [
                {
                    id: 1,
                    name: 'Transfer To/From Seated Positions: Bed to Chair, Chair to Chair, Chair to Exam Table',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_1_notes,
                    flowChart: '_assets/images/algorithms/algorithm-1.jpg',
                    questions: [1, 2, 3, 7, 10, 11],
                    //  ------- spreadsheet answers --------
                    //  1     2    3       10    11        Recommendation
                    //  1B                                 1
                    //  1A    2B                           1
                    //  1A                 10C             1
                    //  1A    2C   3B                      1
                    //  1A    2C                 11B       1
                    //  1A         3B      10A             1
                    //  1A                 10A   11B       1
                    //  1A    2A           10B             2
                    //  1A    2C   3A/3C         11A/11C   3
                    //  1A         3A/3C   10A   11A/11C   3
                    //  ------------------------------------
                    algorithm: [
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            //  ------- spreadsheet answers --------
                            //  1A    2B                           1
                            //  1A                 10C             1
                            //  ------------------------------------
                            id: [1, 2, 10],
                            answerCriteria: '^1A(2B.*|.*10C)$',
                            recommendation: 0
                        }, {
                            //  ------- spreadsheet answers --------
                            //  1A    2C   3B                      1
                            //  1A    2C                 11B       1
                            //  ------------------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C(3B.*|.*11B)$',
                            recommendation: 0
                        }, {
                            //  ------- spreadsheet answers --------
                            //  1A         3B      10A             1
                            //  1A                 10A   11B       1
                            //  ------------------------------------
                            id: [1, 3, 10, 11],
                            answerCriteria: '^1A(3B10A.*|.*10A11B)$',
                            recommendation: 0
                        }, {
                            //  ------- spreadsheet answers --------
                            //  1A    2A           10B             2
                            //  ------------------------------------
                            id: [1, 2, 10],
                            answerCriteria: '^1A2A10B$',
                            recommendation: 1
                        }, {
                            //  ------- spreadsheet answers --------
                            //  1A    2C   3A/3C         11A/11C   3
                            //  1A         3A/3C   10A   11A/11C   3
                            //  ------------------------------------
                            id: [1, 2, 3, 10, 11],
                            answerCriteria: '^1A(2C(3A|3C).*(11A|11C)|.*(3A|3C)10A(11A|11C))$',
                            recommendation: 2
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_1_recommendation_0,
                            equipment: cpaaAlgorithm_1_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_1_recommendation_1,
                            equipment: cpaaAlgorithm_1_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_1_recommendation_2,
                            equipment: cpaaAlgorithm_1_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });


            inject(function (_RecommendationEngine_, _$window_, _$sce_) {
                RecommendationEngine = _RecommendationEngine_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm One", function () {
            //  ------- spreadsheet answers --------
            //  1     2    3       10    11        Recommendation
            //  1B                                 1
            //  1A    2B                           1
            //  1A                 10C             1
            //  1A    2C   3B                      1
            //  1A    2C                 11B       1
            //  1A         3B      10A             1
            //  1A                 10A   11B       1
            //  ------------------------------------
            it("by returning Solution 1 for Algorithm 1 (1A, 2C, 3B, 10C, 11B)", function () {
                var assessment,
                    questions,
                    recommendations;

                //  ------------------------------------
                //  1B                                 1
                //  ------------------------------------
                assessment = {
                    '1' : 'B',
                    '2' : 'A',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A    2B                           1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'B',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A                 10C             1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'A',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'C',
                    '11': 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A    2C   3B                      1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'B',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A    2C                 11B       1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A         3B      10A             1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'A',
                    '3' : 'B',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);

                //  ------------------------------------
                //  1A                 10A   11B       1
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'A',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[0].recommendation);
            });

            //  ------- spreadsheet answers --------
            //  1     2    3       10    11        Recommendation
            //  1A    2A           10B             2
            //  ------------------------------------
            it("by returning Solution 2 for Algorithm 1 (1A, 2A, 10B)", function () {
                var assessment,
                    questions,
                    recommendations;

                //  ------------------------------------
                //  1A    2A           10B             2
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'A',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'B',
                    '11': 'A'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[1].recommendation);
            });

            //  ------- spreadsheet answers --------
            //  1     2    3       10    11        Recommendation
            //  1A    2C   3A/3C         11A/11C   3
            //  1A         3A/3C   10A   11A/11C   3
            //  ------------------------------------
            it("by returning Solution 3 for Algorithm 1 (1A, 2C, 3A/C, 10A, 11A/C)", function () {
                var assessment,
                    questions,
                    recommendations;

                //  ------------------------------------
                //  1A    2C   3A/3C         11A/11C   3
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);

                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[2].recommendation);

                //  ------------------------------------
                //  1A    2C   3A/3C         11A/11C   3
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'C',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'C'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);

                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[2].recommendation);


                //  ------------------------------------
                //  1A    2C   3A/3C         11A/11C   3
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'C',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'C'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);

                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[2].recommendation);

                //  ------------------------------------
                //  1A    2C   3A/3C         11A/11C   3
                //  ------------------------------------
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'C'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);

                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(ConstantsMock.algorithms[0].solution[2].recommendation);
            });

            it("by flagging Bariatric", function () {
                var assessment,
                    questions,
                    recommendations;
                assessment = {
                    '1' : 'A',
                    '2' : 'C',
                    '3' : 'A',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '10': 'A',
                    '11': 'A'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].bariatricFlag).toEqual(true);


                assessment = {
                    '1' : 'A',
                    '2' : 'B',
                    '3' : 'C',
                    '7' : 'B', // Bariatric Flag.  A = Yes.
                    '10': 'C',
                    '11': 'B'
                };
                questions = ConstantsMock.algorithms;
                recommendations = RecommendationEngine.recommend(assessment, questions);
                expect(recommendations[0].bariatricFlag).toEqual(false);
            });
            it("by giving totally wrong answers", function () {
                var assessment,
                    questions,
                    recommendations;
                assessment = {
                    '1' : 'Z',
                    '2' : 'Z',
                    '3' : 'Z',
                    '7' : 'Z', // Bariatric Flag.  A = Yes.
                    '10': 'Z',
                    '11': 'Z'
                };
                questions = ConstantsMock.algorithms;

                recommendations = RecommendationEngine.recommend(assessment, questions);

                expect(recommendations).toEqual([]);
            });
        });
    });
});