/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_recommendation_3.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_recommendation_4.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-10/algorithm_10_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_10_notes,
             incomingCpaaAlgorithm_10_recommendation_0,
             incomingCpaaAlgorithm_10_recommendation_1,
             incomingCpaaAlgorithm_10_recommendation_2,
             incomingCpaaAlgorithm_10_recommendation_3,
             incomingCpaaAlgorithm_10_recommendation_4,
             incomingCpaaAlgorithm_10_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_10_notes               = incomingCpaaAlgorithm_10_notes.toString(),
            cpaaAlgorithm_10_recommendation_0    = incomingCpaaAlgorithm_10_recommendation_0.toString(),
            cpaaAlgorithm_10_recommendation_1    = incomingCpaaAlgorithm_10_recommendation_1.toString(),
            cpaaAlgorithm_10_recommendation_2    = incomingCpaaAlgorithm_10_recommendation_2.toString(),
            cpaaAlgorithm_10_recommendation_3    = incomingCpaaAlgorithm_10_recommendation_3.toString(),
            cpaaAlgorithm_10_recommendation_4    = incomingCpaaAlgorithm_10_recommendation_4.toString(),
            cpaaAlgorithm_10_equipment_0         = incomingCpaaAlgorithm_10_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 10,
                    name: 'Ambulation',
                    flowChart: '_assets/images/algorithms/algorithm-10.jpg',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_10_notes,
                    questions: [1, 2, 3, 7, 10, 11, 12, 17],
                    //  -------------------------------------------------------------
                    //  1   2   3       10    11        12                    17    Recommendation
                    //  1B                                                          1
                    //  1A  2B                                                      2
                    //  1A              10C                                         2
                    //  1A                                                    17B   2
                    //  1A  2A          10B             12F                   17A   3
                    //  1A  2A  3B      10B             12A/12B/12C/12D/12E   17A   4
                    //  1A  2A          10B   11B       12A/12B/12C/12D/12E   17A   4
                    //  1A  2C  3B                                                  4
                    //  1A  2C                11B                                   4
                    //  1A  2A  3A/3C   10B   11A/11C   12A/12B/12C/12D/12E   17A   5
                    //  1A  2C  3A/3C         11A/11C                               5
                    //  1A  2A  3A      10A   11A       12A                   17A   5
                    //  -------------------------------------------------------------

                    algorithm: [
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------------------------------
                            //  1B                                                          1
                            //  -------------------------------------------------------------
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2B                                                      2
                            //  1A              10C                                         2
                            //  1A                                                    17B   2
                            //  -------------------------------------------------------------
                            id: [1, 2, 10, 17],
                            answerCriteria: '^1A(2B.*|.*10C.*|.*17B)$',
                            recommendation: 1
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2A          10B             12F                   17A   3
                            //  -------------------------------------------------------------
                            id: [1, 2, 10, 12, 17],
                            answerCriteria: '^1A2A10B12F17A$',
                            recommendation: 2
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2A  3B      10B             12A/12B/12C/12D/12E   17A   4
                            //  1A  2A          10B   11B       12A/12B/12C/12D/12E   17A   4
                            //  -------------------------------------------------------------
                            id: [1, 2, 3, 10, 11, 12, 17],
                            answerCriteria: '^1A2A(3B10B.*|.*10B11B)12(A|B|C|D|E)17A$',
                            recommendation: 3
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2C  3B                                                  4
                            //  1A  2C                11B                                   4
                            //  -------------------------------------------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C(3B.*|.*11B)$',
                            recommendation: 3
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2A  3A/3C   10B   11A/11C   12A/12B/12C/12D/12E   17A   5
                            //  -------------------------------------------------------------
                            id: [1, 2, 3, 10, 11, 12, 17],
                            answerCriteria: '^1A2A3(A|C)10B11(A|C)12(A|B|C|D|E)17A$',
                            recommendation: 4
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2C  3A/3C         11A/11C                               5
                            //  -------------------------------------------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C3(A|C)11(A|C)$',
                            recommendation: 4
                        }, {
                            //  -------------------------------------------------------------
                            //  1A  2A  3A      10A   11A       12A                   17A   5
                            //  -------------------------------------------------------------
                            id: [1, 2, 3, 10, 11, 12, 17],
                            answerCriteria: '^1A2A3A10A11A12A17A$',
                            recommendation: 4
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_10_recommendation_0,
                            equipment: cpaaAlgorithm_10_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_10_recommendation_1,
                            equipment: cpaaAlgorithm_10_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_10_recommendation_2,
                            equipment: cpaaAlgorithm_10_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_10_recommendation_3,
                            equipment: cpaaAlgorithm_10_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_10_recommendation_4,
                            equipment: cpaaAlgorithm_10_equipment_0             // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations for Algorithm Ten", function () {
            //  -------------------------------------------------------------
            //  1   2   3       10    11        12                    17    Recommendation
            //  1B                                                          1
            //  1A  2B                                                      2
            //  1A              10C                                         2
            //  1A                                                    17B   2
            //  1A  2A          10B             12F                   17A   3
            //  1A  2A  3B      10B             12A/12B/12C/12D/12E   17A   4
            //  1A  2A          10B   11B       12A/12B/12C/12D/12E   17A   4
            //  1A  2C  3B                                                  4
            //  1A  2C                11B                                   4
            //  1A  2A  3A/3C   10B   11A/11C   12A/12B/12C/12D/12E   17A   5
            //  1A  2C  3A/3C         11A/11C                               5
            //  1A  2A  3A      10A   11A       12A                   17A   5
            //  -------------------------------------------------------------

            it("by returning Solution 1", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[9]];

                //  -------------------------------------------------------------
                //  1B                                                          1
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'B',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'B',
                    '10' : 'A',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[0].recommendation);
            });

            it("by returning Solution 2", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[9]];

                //  -------------------------------------------------------------
                //  1A  2B                                                      2
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'B',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[1].recommendation);

                //  -------------------------------------------------------------
                //  1A              10C                                         2
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'C',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[1].recommendation);

                //  -------------------------------------------------------------
                //  1A                                                    17B   2
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[1].recommendation);
            });

            it("by returning Solution 3", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[9]];

                //  -------------------------------------------------------------
                //  1A  2A          10B             12F                   17A   3
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'B',
                    '11' : 'A',
                    '12' : 'F',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[2].recommendation);
            });

            it("by returning Solution 4", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[9]];

                //  -------------------------------------------------------------
                //  1A  2A  3B      10B             12A/12B/12C/12D/12E   17A   4
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'B',
                    '7'  : 'A',
                    '10' : 'B',
                    '11' : 'A',
                    '12' : 'B',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[3].recommendation);

                //  -------------------------------------------------------------
                //  1A  2A          10B   11B       12A/12B/12C/12D/12E   17A   4
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'B',
                    '11' : 'B',
                    '12' : 'B',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[3].recommendation);

                //  -------------------------------------------------------------
                //  1A  2C  3B                                                  4
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'B',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[3].recommendation);

                //  -------------------------------------------------------------
                //  1A  2C                11B                                   4
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'B',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[3].recommendation);
            });

            it("by returning Solution 5", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[9]];

                //  -------------------------------------------------------------
                //  1A  2A  3A/3C   10B   11A/11C   12A/12B/12C/12D/12E   17A   5
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'C',
                    '7'  : 'A',
                    '10' : 'B',
                    '11' : 'C',
                    '12' : 'C',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[4].recommendation);

                //  -------------------------------------------------------------
                //  1A  2C  3A/3C         11A/11C                               5
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[4].recommendation);

                //  -------------------------------------------------------------
                //  1A  2A  3A      10A   11A       12A                   17A   5
                //  -------------------------------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A',
                    '10' : 'A',
                    '11' : 'A',
                    '12' : 'A',
                    '17' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[9].solution[4].recommendation);
            });
        });
    });
});