/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-11/algorithm_11_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-11/algorithm_11_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-11/algorithm_11_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-11/algorithm_11_equipment_0.html'
], function (RecommendationEngine,
             Constants,
             incomingCpaaAlgorithm_11_notes,
             incomingCpaaAlgorithm_11_recommendation_0,
             incomingCpaaAlgorithm_11_recommendation_1,
             incomingCpaaAlgorithm_11_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_11_notes = incomingCpaaAlgorithm_11_notes.toString(),
            cpaaAlgorithm_11_recommendation_0 = incomingCpaaAlgorithm_11_recommendation_0.toString(),
            cpaaAlgorithm_11_recommendation_1 = incomingCpaaAlgorithm_11_recommendation_1.toString(),
            cpaaAlgorithm_11_equipment_0 = incomingCpaaAlgorithm_11_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 11,
                    name: 'Patient Handling Task Requiring Lifting of Extremities',
                    flowChart: '_assets/images/algorithms/algorithm-11.jpg',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_11_notes,
                    questions: [1, 6, 7],
                    //  ------------
                    //  1    6     Recommendation
                    //  1B         1
                    //  1A   6B    1
                    //  1A   6A    2
                    //  ------------
                    algorithm: [
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  ------------
                            //  1B         1
                            //  ------------
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            //  ------------
                            //  1A   6B    1
                            //  ------------
                            id: [1, 6],
                            answerCriteria: '^1A6B$',
                            recommendation: 0
                        }, {
                            //  ------------
                            //  1A   6A    2
                            //  ------------
                            id: [1, 6],
                            answerCriteria: '^1A6A$',
                            recommendation: 1
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_11_recommendation_0,
                            equipment: cpaaAlgorithm_11_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_11_recommendation_1,
                            equipment: cpaaAlgorithm_11_equipment_0             // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Eleven", function () {
            //  ------------
            //  1    6     Recommendation
            //  1B         1
            //  1A   6B    1
            //  1A   6A    2
            //  ------------
            it("by returning Solution 1", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[10]];

                //  ------------
                //  1B         1
                //  ------------
                assessment = {
                    '1': 'B',
                    '6': 'A',
                    '7': 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[10].solution[0].recommendation);

                //  ------------
                //  1B         1
                //  ------------
                assessment = {
                    '1': 'B',
                    '6': 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[10].solution[0].recommendation);

                //  ------------
                //  1A   6B    1
                //  ------------
                assessment = {
                    '1': 'A',
                    '6': 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[10].solution[0].recommendation);
            });

            it("by returning Solution 2", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[10]];

                //  ------------
                //  1A   6A    2
                //  ------------
                assessment = {
                    '1': 'A',
                    '6': 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[10].solution[1].recommendation);
            });

        });
    });
});