/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-13/algorithm_13_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-13/algorithm_13_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-13/algorithm_13_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-13/algorithm_13_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_13_notes,
             incomingCpaaAlgorithm_13_recommendation_0,
             incomingCpaaAlgorithm_13_recommendation_1,
             incomingCpaaAlgorithm_13_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_13_notes               = incomingCpaaAlgorithm_13_notes.toString(),
            cpaaAlgorithm_13_recommendation_0    = incomingCpaaAlgorithm_13_recommendation_0.toString(),
            cpaaAlgorithm_13_recommendation_1    = incomingCpaaAlgorithm_13_recommendation_1.toString(),
            cpaaAlgorithm_13_equipment_0         = incomingCpaaAlgorithm_13_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 13,
                    name: 'Bariatric Patient Handling Task Requiring Access to Perineal Area',
                    flowChart: '_assets/images/algorithms/algorithm-13.jpg',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_13_notes,
                    questions: [1, 7, 15],
                    //  ------------
                    //  1    15    Recommendation
                    //  1B         1
                    //  1A   15B   1
                    //  1A   15A   2
                    //  ------------
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  ------------
                            //  1B         1
                            //  1A   15B   1
                            //  ------------
                            id: [1, 15],
                            answerCriteria: '^(1B.*|1A15B)$',
                            recommendation: 0
                        }, {
                            //  ------------
                            //  1A   15A   2
                            //  ------------
                            id: [1, 15],
                            answerCriteria: '^1A15A$',
                            recommendation: 1
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_13_recommendation_0,
                            equipment: cpaaAlgorithm_13_equipment_0             // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_13_recommendation_1,
                            equipment: cpaaAlgorithm_13_equipment_0             // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Thirteen", function () {
            //  ------------
            //  1    15    Recommendation
            //  1B         1
            //  1A   15B   1
            //  1A   15A   2
            //  ------------
            it("by returning Solution 1 for Algorithm 13", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[12]];

                //  ------------
                //  1B         1
                //  ------------
                assessment = {
                    '1'  : 'B',
                    '15' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[12].solution[0].recommendation);

                //  ------------
                //  1B         1
                //  ------------
                assessment = {
                    '1'  : 'B',
                    '15' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[12].solution[0].recommendation);

                //  ------------
                //  1A   15B   1
                //  ------------
                assessment = {
                    '1'  : 'A',
                    '15' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[12].solution[0].recommendation);
            });

            it("by returning Solution 2 for Algorithm 13", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = [Constants.algorithms[12]];

                //  ------------
                //  1A   15A   2
                //  ------------
                assessment = {
                    '1'  : 'A',
                    '15' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[12].solution[1].recommendation);
            });
        });
    });
});