/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_2_notes,
             incomingCpaaAlgorithm_2_recommendation_0,
             incomingCpaaAlgorithm_2_recommendation_1,
             incomingCpaaAlgorithm_2_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_2_notes               = incomingCpaaAlgorithm_2_notes.toString(),
            cpaaAlgorithm_2_recommendation_0    = incomingCpaaAlgorithm_2_recommendation_0.toString(),
            cpaaAlgorithm_2_recommendation_1    = incomingCpaaAlgorithm_2_recommendation_1.toString(),
            cpaaAlgorithm_2_equipment_0         = incomingCpaaAlgorithm_2_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 2,
                    name: 'Lateral Transfer to/from Supine Positions: Bed, Stretcher, Trolley, Procedure Table',
                    bariatricFlag: false,
                    flowChart: 'path/to/chart/image.png',
                    notes: cpaaAlgorithm_2_notes,
                    questions: [1, 4, 7],
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '^7A',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1B.*',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1A4B',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1A4A',
                            recommendation: 1
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_2_recommendation_0,
                            equipment: cpaaAlgorithm_2_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_2_recommendation_1,
                            equipment: cpaaAlgorithm_2_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Two", function () {
            // -------------------------------------------------------------------------------------------------------------
            // -    Algorithm Two
            // -------------------------------------------------------------------------------------------------------------
            it("by returning Solution 1 for Algorithm 2", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // ABB - Recommendation 2
                assessment = {
                    '1' : 'A',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'B',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[0].recommendation);

                // BAB - Recommendation 2
                assessment = {
                    '1' : 'B',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'A',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[0].recommendation);

                // BAA - Recommendation 2
                assessment = {
                    '1' : 'B',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'A',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'A'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[0].recommendation);

                // BBB - Recommendation 2
                assessment = {
                    '1' : 'B',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'B',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[0].recommendation);

                // BBA - Recommendation 2
                assessment = {
                    '1' : 'B',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'B',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'A'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[0].recommendation);
            });
            it("by returning Solution 2 for Algorithm 2", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // AAB - Recommendation 2
                assessment = {
                    '1' : 'A',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'A',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[1].recommendation);

                // AAA - Recommendation 2
                assessment = {
                    '1' : 'A',  // 1B flags solution 0 regardless of answer 4
                    '4' : 'A',  // IF 1A, then 4B will flag Solution 0
                    '7' : 'A'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[1].solution[1].recommendation);
            });
        });
    });
});