/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-4/algorithm_4_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-4/algorithm_4_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-4/algorithm_4_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-4/algorithm_4_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-4/algorithm_4_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_4_notes,
             incomingCpaaAlgorithm_4_recommendation_0,
             incomingCpaaAlgorithm_4_recommendation_1,
             incomingCpaaAlgorithm_4_recommendation_2,
             incomingCpaaAlgorithm_4_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_4_notes               = incomingCpaaAlgorithm_4_notes.toString(),
            cpaaAlgorithm_4_recommendation_0    = incomingCpaaAlgorithm_4_recommendation_0.toString(),
            cpaaAlgorithm_4_recommendation_1    = incomingCpaaAlgorithm_4_recommendation_1.toString(),
            cpaaAlgorithm_4_recommendation_2    = incomingCpaaAlgorithm_4_recommendation_2.toString(),
            cpaaAlgorithm_4_equipment_0         = incomingCpaaAlgorithm_4_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 4,
                    name: 'Reposition in Chair: Wheelchair, Dependency Chair or Other Chair',
                    flowChart: 'path/to/chart/image.png',
                    notes: cpaaAlgorithm_4_notes,
                    questions: [1, 7, 16],
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '7A',
                            recommendation: 0
                        }, {
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            id: [1, 16],
                            answerCriteria: '^1A16B$',
                            recommendation: 0
                        }, {
                            id: [1, 16],
                            answerCriteria: '^1A16C$',
                            recommendation: 2
                        }, {
                            id: [1, 16],
                            answerCriteria: '^1A16A$',
                            recommendation: 1
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_4_recommendation_0,
                            equipment: cpaaAlgorithm_4_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_4_recommendation_1,
                            equipment: cpaaAlgorithm_4_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_4_recommendation_2,
                            equipment: cpaaAlgorithm_4_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Four", function () {
            // -------------------------------------------------------------------------------------------------------------
            // -    Algorithm Two
            // -------------------------------------------------------------------------------------------------------------
            it("by returning Solution 1 for Algorithm 4", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // ABB - Recommendation 2
                //  Questions   1   16      Recommendation
                //              1B          1
                //              1A  16B     1
                //              1A  16A     2
                //              1A  16C     3
                assessment = {
                    '1'  : 'B',
                    '16' : 'B',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[3].solution[0].recommendation);

                assessment = {
                    '1'  : 'B',
                    '16' : 'A',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[3].solution[0].recommendation);

                // BAB - Recommendation 2
                assessment = {
                    '1'  : 'A',
                    '16' : 'B',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[3].solution[0].recommendation);
            });
            it("by returning Solution 2 for Algorithm 4", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // AAB - Recommendation 2
                //  Questions   1   16      Recommendation
                //              1B          1
                //              1A  16B     1
                //              1A  16A     2
                //              1A  16C     3
                assessment = {
                    '1'  : 'A',
                    '16' : 'A',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[3].solution[1].recommendation);
            });
            it("by returning Solution 3 for Algorithm 4", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // AAB - Recommendation 2
                //  Questions   1   16      Recommendation
                //              1B          1
                //              1A  16B     1
                //              1A  16A     2
                //              1A  16C     3
                assessment = {
                    '1'  : 'A',
                    '16' : 'C',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[3].solution[2].recommendation);
            });
        });
    });
});