/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-5/algorithm_5_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-5/algorithm_5_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-5/algorithm_5_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-5/algorithm_5_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-5/algorithm_5_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_5_notes,
             incomingCpaaAlgorithm_5_recommendation_0,
             incomingCpaaAlgorithm_5_recommendation_1,
             incomingCpaaAlgorithm_5_recommendation_2,
             incomingCpaaAlgorithm_5_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_5_notes               = incomingCpaaAlgorithm_5_notes.toString(),
            cpaaAlgorithm_5_recommendation_0    = incomingCpaaAlgorithm_5_recommendation_0.toString(),
            cpaaAlgorithm_5_recommendation_1    = incomingCpaaAlgorithm_5_recommendation_1.toString(),
            cpaaAlgorithm_5_recommendation_2    = incomingCpaaAlgorithm_5_recommendation_2.toString(),
            cpaaAlgorithm_5_equipment_0         = incomingCpaaAlgorithm_5_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 5,
                    name: 'Transport in Bed/Stretcher/Wheelchair',
                    flowChart: '/_assets/images/algorithms/algorithm-5.jpg',
                    notes: cpaaAlgorithm_5_notes,
                    questions: [1, 7],
                    //  From Spreadsheet.
                    //  1	7	Recommendation
                    //  1B		1
                    //  1A	7A	3
                    //  1A	7B	2
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '7A',
                            recommendation: 0
                        }, {
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            id: [1, 7],
                            answerCriteria: '^1A7B$',
                            recommendation: 1
                        }, {
                            id: [1, 7],
                            answerCriteria: '^1A7A$',
                            recommendation: 2
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_5_recommendation_0,
                            equipment: cpaaAlgorithm_5_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_5_recommendation_1,
                            equipment: cpaaAlgorithm_5_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_5_recommendation_2,
                            equipment: cpaaAlgorithm_5_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Five", function () {
            // -------------------------------------------------------------------------------------------------------------
            // -    Algorithm Two
            // -------------------------------------------------------------------------------------------------------------
            it("by returning Solution 1 for Algorithm 5", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                // Recommendation 2
                //  1	7	Recommendation
                //  1B		1
                //  1A	7A	3
                //  1A	7B	2
                assessment = {
                    '1'  : 'B',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[4].solution[0].recommendation);

                assessment = {
                    '1'  : 'B',
                    '7'  : 'A'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[4].solution[0].recommendation);
            });
            it("by returning Solution 2 for Algorithm 5", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                assessment = {
                    '1'  : 'A',
                    '7'  : 'B'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[4].solution[1].recommendation);
            });
            it("by returning Solution 3 for Algorithm 4", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                assessment = {
                    '1'  : 'A',
                    '7'  : 'A'   // Bariatric Flag.  A = Yes.
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[4].solution[2].recommendation);
            });
        });
    });
});