/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-7/algorithm_7_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-7/algorithm_7_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-7/algorithm_7_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-7/algorithm_7_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-7/algorithm_7_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_7_notes,
             incomingCpaaAlgorithm_7_recommendation_0,
             incomingCpaaAlgorithm_7_recommendation_1,
             incomingCpaaAlgorithm_7_recommendation_2,
             incomingCpaaAlgorithm_7_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_7_notes               = incomingCpaaAlgorithm_7_notes.toString(),
            cpaaAlgorithm_7_recommendation_0    = incomingCpaaAlgorithm_7_recommendation_0.toString(),
            cpaaAlgorithm_7_recommendation_1    = incomingCpaaAlgorithm_7_recommendation_1.toString(),
            cpaaAlgorithm_7_recommendation_2    = incomingCpaaAlgorithm_7_recommendation_2.toString(),
            cpaaAlgorithm_7_equipment_0         = incomingCpaaAlgorithm_7_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 7,
                    name: 'Showering and Bathing',
                    flowChart: '_assets/images/algorithms/algorithm-7.jpg',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_7_notes,
                    questions: [1, 2, 3, 7, 10, 11],
                    algorithm: [
                        //  -------------------------------------
                        //  1     2     3       10     11       Recommendation
                        //  1B                                  1
                        //  1A    2B                            1
                        //  1A                  10C             1
                        //  1A    2C    3B                      1
                        //  1A    2C                   11B      1
                        //  1A          3B      10A             1
                        //  1A                  10A    11B      1
                        //  1A    2A            10B             2
                        //  1A    2C    3A/3C          11A/11C  3
                        //  1A          3A/3C   10A    11A/11C  3
                        //  -------------------------------------
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------
                            //  1B                                  1
                            //  -------------------------------------
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------
                            //  1A    2B                            1
                            //  1A                  10C             1
                            //  -------------------------------------
                            id: [1, 2, 10],
                            answerCriteria: '^1A(2B.*|.*10C)$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------
                            //  1A    2C    3B                      1
                            //  1A    2C                   11B      1
                            //  -------------------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C(3B.*|.*11B)$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------
                            //  1A          3B      10A             1
                            //  1A                  10A    11B      1
                            //  -------------------------------------
                            id: [1, 3, 10, 11],
                            answerCriteria: '^1A(3B10A.*|.*10A11B)$',
                            recommendation: 0
                        }, {
                            //  -------------------------------------
                            //  1A    2A            10B             2
                            //  -------------------------------------
                            id: [1, 2, 10],
                            answerCriteria: '^1A2A10B$',
                            recommendation: 1
                        }, {
                            //  -------------------------------------
                            //  1A    2C    3A/3C          11A/11C  3
                            //  1A          3A/3C   10A    11A/11C  3
                            //  -------------------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C(3A|3C)(11A|11C)$',
                            recommendation: 2
                        }, {
                            //  -------------------------------------
                            //  1A    2C    3A/3C          11A/11C  3
                            //  1A          3A/3C   10A    11A/11C  3
                            //  -------------------------------------
                            id: [1, 3, 10, 11],
                            answerCriteria: '^1A(3A|3C)10A(11A|11C)$',
                            recommendation: 2
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_7_recommendation_0,
                            equipment: cpaaAlgorithm_7_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_7_recommendation_1,
                            equipment: cpaaAlgorithm_7_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_7_recommendation_2,
                            equipment: cpaaAlgorithm_7_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });
        describe("should test the recommendations for Algorithm Seven", function () {
            // ---------------------------------------------------------------------------------------------------------
            // -    Algorithm Seven
            // ---------------------------------------------------------------------------------------------------------

            //  -------------------------------------
            //  1     2     3       10     11       Recommendation
            //  1B                                  1
            //  1A    2B                            1
            //  1A                  10C             1
            //  1A    2C    3B                      1
            //  1A    2C                   11B      1
            //  1A          3B      10A             1
            //  1A                  10A    11B      1
            //  1A    2A            10B             2
            //  1A    2C    3A/3C          11A/11C  3
            //  1A          3A/3C   10A    11A/11C  3
            //  -------------------------------------
            it("by returning Solution 1", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  -------------------------------------
                //  1B                                  1
                //  -------------------------------------
                assessment = {
                    '1'  : 'B',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A    2B                            1
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'B',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'B',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A                  10C             1
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'C',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A    2C    3B                      1
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'B',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'B',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A    2C                   11B      1
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '7'  : 'B', // Bariatric Flag.  A = Yes.
                    '10' : 'C',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(false);

                //  -------------------------------------
                //  1A          3B      10A             1
                //  -------------------------------------
                //  1A                  10A    11B      1
                //  1A    2A            10B             2
                //  1A    2C    3A/3C          11A/11C  3
                //  1A          3A/3C   10A    11A/11C  3
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'B',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A                  10A    11B      1
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);
            });

            it("by returning Solution 2", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  -------------------------------------
                //  1A    2A            10B             2
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'C',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'B',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[1].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);
            });

            it("by returning Solution 3 for Algorithm 6", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  -------------------------------------
                //  1A    2C    3A/3C          11A/11C  3
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[2].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  -------------------------------------
                //  1A          3A/3C   10A    11A/11C  3
                //  -------------------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'C',
                    '7'  : 'B', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[6].solution[2].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(false);

            });
        });
    });
});