/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_recommendation_3.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-8/algorithm_8_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_8_notes,
             incomingCpaaAlgorithm_8_recommendation_0,
             incomingCpaaAlgorithm_8_recommendation_1,
             incomingCpaaAlgorithm_8_recommendation_2,
             incomingCpaaAlgorithm_8_recommendation_3,
             incomingCpaaAlgorithm_8_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_8_notes               = incomingCpaaAlgorithm_8_notes.toString(),
            cpaaAlgorithm_8_recommendation_0    = incomingCpaaAlgorithm_8_recommendation_0.toString(),
            cpaaAlgorithm_8_recommendation_1    = incomingCpaaAlgorithm_8_recommendation_1.toString(),
            cpaaAlgorithm_8_recommendation_2    = incomingCpaaAlgorithm_8_recommendation_2.toString(),
            cpaaAlgorithm_8_recommendation_3    = incomingCpaaAlgorithm_8_recommendation_3.toString(),
            cpaaAlgorithm_8_equipment_0         = incomingCpaaAlgorithm_8_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 8,
                    name: 'Floor/Fall Recovery',
                    flowChart: '_assets/images/algorithms/algorithm-8.jpg',
                    bariatricFlag: false,
                    notes: cpaaAlgorithm_8_notes,
                    questions: [1, 7, 8, 9, 13],
                    //  ----------------------
                    //  1    8    9    13    Recommendation
                    //       8A   9B         1
                    //  1B   8A   9A         2
                    //  1B   8B              2
                    //  1A   8A   9A   13B   2
                    //  1A   8B        13B   2
                    //  1A   8A   9A   13A   3
                    //  1A   8B        13A   3
                    //       8C   9C   13C   4
                    //  ----------------------
                    algorithm: [
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  ----------------------
                            //       8A   9B         1
                            //  ----------------------
                            id: [8, 9],
                            answerCriteria: '^8A9B$',
                            recommendation: 0
                        }, {
                            //  ----------------------
                            //  1B   8A   9A         2
                            //  1B   8B              2
                            //  ----------------------
                            id: [1, 8, 9],
                            answerCriteria: '^1B(8B.*|8A9A)$',
                            recommendation: 1
                        }, {
                            //  ----------------------
                            //  1A   8A   9A   13B   2
                            //  1A   8B        13B   2
                            //  ----------------------
                            id: [1, 8, 9, 13],
                            answerCriteria: '^1A(8A9A13B|8B.*13B)$',
                            recommendation: 1
                        }, {
                            //  ----------------------
                            //  1A   8A   9A   13A   3
                            //  1A   8B        13A   3
                            //  ----------------------
                            id: [1, 8, 9, 13],
                            answerCriteria: '^1A(8A9A13A|8B.*13A)$',
                            recommendation: 2
                        }, {
                            //  ----------------------
                            //       8C   9C   13C   4
                            //  ----------------------
                            id: [8, 9, 13],
                            answerCriteria: '^.*8C9C13C$',
                            recommendation: 3
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_8_recommendation_0,
                            equipment: cpaaAlgorithm_8_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_8_recommendation_1,
                            equipment: cpaaAlgorithm_8_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_8_recommendation_2,
                            equipment: cpaaAlgorithm_8_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_8_recommendation_3,
                            equipment: cpaaAlgorithm_8_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };

        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations for Algorithm Eight", function () {
            //  ----------------------
            //  1    8    9    13    Recommendation
            //       8A   9B         1
            //  1B   8A   9A         2
            //  1B   8B              2
            //  1A   8A   9A   13B   2
            //  1A   8B        13B   2
            //  1A   8A   9A   13A   3
            //  1A   8B        13A   3
            //  ----------------------
            it("by returning Solution 1 for Algorithm 8", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------
                //       8A   9B         1
                //  ----------------------
                assessment = {
                    '8'  : 'A',
                    '7'  : 'B',   // Bariatric Flag.  A = Yes.
                    '9'  : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(false);
            });

            it("by returning Solution 2 for Algorithm 8", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------
                //  1B   8A   9A         2
                //  ----------------------
                assessment = {
                    '1'  : 'B',
                    '8'  : 'A',
                    '9'  : 'A',
                    '7'  : 'B', // Bariatric Flag.  A = Yes.
                    '13' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[1].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(false);

                //  ----------------------
                //  1B   8A   9A         2
                //  1B   8B              2
                //  ----------------------
                assessment = {
                    '1'  : 'B',
                    '8'  : 'B',
                    '9'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '13' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[1].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  ----------------------
                //  1A   8A   9A   13B   2
                //  ----------------------
                assessment = {
                    '1'  : 'A',
                    '8'  : 'A',
                    '9'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '13' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[1].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  ----------------------
                //  1A   8B        13B   2
                //  ----------------------
                assessment = {
                    '1'  : 'A',
                    '8'  : 'B',
                    '9'  : 'B',
                    '7'  : 'B', // Bariatric Flag.  A = Yes.
                    '13' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[1].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(false);
            });

            it("by returning Solution 3 for Algorithm 8", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------
                //  1A   8A   9A   13A   3
                //  ----------------------
                assessment = {
                    '1'  : 'A',
                    '8'  : 'A',
                    '9'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '13' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[2].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  ----------------------
                //  1A   8B        13A   3
                //  ----------------------
                assessment = {
                    '1'  : 'A',
                    '8'  : 'B',
                    '9'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '13' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[2].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);
            });

            it("by returning Solution 3 for Algorithm 8", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------
                //       8C   9C   13C   4
                //  ----------------------
                assessment = {
                    '1' : 'A',
                    '8' : 'C',
                    '9' : 'C',
                    '7' : 'A', // Bariatric Flag.  A = Yes.
                    '13': 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[7].solution[3].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);
            });
        });
    });
});