/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'Constants',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_recommendation_3.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-9/algorithm_9_equipment_0.html'
], function (RecommendationEngine,
             Constants,

             incomingCpaaAlgorithm_9_notes,
             incomingCpaaAlgorithm_9_recommendation_0,
             incomingCpaaAlgorithm_9_recommendation_1,
             incomingCpaaAlgorithm_9_recommendation_2,
             incomingCpaaAlgorithm_9_recommendation_3,
             incomingCpaaAlgorithm_9_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            $sce,

            cpaaAlgorithm_9_notes               = incomingCpaaAlgorithm_9_notes.toString(),
            cpaaAlgorithm_9_recommendation_0    = incomingCpaaAlgorithm_9_recommendation_0.toString(),
            cpaaAlgorithm_9_recommendation_1    = incomingCpaaAlgorithm_9_recommendation_1.toString(),
            cpaaAlgorithm_9_recommendation_2    = incomingCpaaAlgorithm_9_recommendation_2.toString(),
            cpaaAlgorithm_9_recommendation_3    = incomingCpaaAlgorithm_9_recommendation_3.toString(),
            cpaaAlgorithm_9_equipment_0         = incomingCpaaAlgorithm_9_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            algorithms: [
                {
                    id: 9,
                    name: 'Transfer between Vehicle and Wheelchair, Powered Wheelchair, or Stretcher',
                    bariatricFlag: false,
                    flowChart: '_assets/images/algorithms/algorithm-9.jpg',
                    notes: cpaaAlgorithm_9_notes,
                    questions: [1, 2, 3, 7, 10, 11],
                    //  ----------------------------
                    //  1    2    3    10    11    Recommendation
                    //  1B                         1
                    //  1A   2B                    1
                    //  1A             10C         1
                    //  1A   2C   3B               1
                    //  1A   2C              11B   1
                    //  1A        3B   10A         1
                    //  1A             10A   11B   1
                    //  1A   2A        10B         2
                    //  1A   2C   3A         11A   3
                    //  1A        3A   10A   11A   3
                    //  1A   2C   3C         11C   4
                    //  1A        3C   10A   11C   4
                    //  ----------------------------
                    algorithm: [
                        {
                            // 7 is always standalone first check.
                            // For Bariatric only; we ignore any found solution
                            // for solo-check of 7.
                            id: [7],
                            answerCriteria: '^7A$',
                            recommendation: 0
                        }, {
                            //  ----------------------------
                            //  1B                         1
                            //  ----------------------------
                            id: [1],
                            answerCriteria: '^1B$',
                            recommendation: 0
                        }, {
                            //  ----------------------------
                            //  1A   2B                    1
                            //  1A        10C              1
                            //  ----------------------------
                            id: [1, 2, 3, 10],
                            answerCriteria: '^1A(2B.*|.*10C|2C3B.*)',
                            recommendation: 0
                        }, {
                            //  ----------------------------
                            //  1A   2C   3B               1
                            //  1A   2C              11B   1
                            //  ----------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A2C(3B.*|.*11B)$',
                            recommendation: 0
                        }, {
                            //  ----------------------------
                            //  1A        3B   10A         1
                            //  1A             10A   11B   1
                            //  ----------------------------
                            id: [1, 3, 10, 11],
                            answerCriteria: '^1A(3B10A.*|.*10A11B)$',
                            recommendation: 0
                        }, {
                            //  ----------------------------
                            //  1A   2A        10B         2
                            //  ----------------------------
                            id: [1, 2, 10],
                            answerCriteria: '^1A2A10B$',
                            recommendation: 1
                        }, {
                            //  ----------------------------
                            //  1A   2C   3A         11A   3
                            //  1A        3A   10A   11A   3
                            //  ----------------------------
                            id: [1, 2, 3, 10, 11],
                            answerCriteria: '^1A(2C3A.*|.*3A10A)11A$',
                            recommendation: 2
                        }, {
                            //  ----------------------------
                            //  1A   2C   3C         11C   4
                            //  1A        3C   10A   11C   4
                            //  ----------------------------
                            id: [1, 2, 3, 11],
                            answerCriteria: '^1A(2C3C.*|.*3C10A)11C$',
                            recommendation: 3
                        }, {
                            //  ----------------------------
                            //  1A   2C   3A   10A   11C   4
                            //  1A   2C   3C   10A   11A   4
                            //  ----------------------------
                            id: [1, 2, 3, 10, 11],
                            answerCriteria: '^1A2C(3A10A11C|3C10A11A)$',
                            recommendation: 3
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_9_recommendation_0,
                            equipment: cpaaAlgorithm_9_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_9_recommendation_1,
                            equipment: cpaaAlgorithm_9_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_9_recommendation_2,
                            equipment: cpaaAlgorithm_9_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_9_recommendation_3,
                            equipment: cpaaAlgorithm_9_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                // $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });

            inject(function (_RecommendationEngine_, _$window_, _$sce_, _Constants_) {
                RecommendationEngine = _RecommendationEngine_;
                Constants = _Constants_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the recommendations - Algorithm Nine", function () {
            //  ----------------------------
            //  1    2    10    3    11    Recommendation
            //  1B                         1
            //  1A   2B                    1
            //  1A             10C         1
            //  1A   2C   3B               1
            //  1A   2C              11B   1
            //  1A        3B   10A         1
            //  1A             10A   11B   1
            //  1A   2A        10B         2
            //  1A   2C   3A         11A   3
            //  1A        3A   10A   11A   3
            //  1A   2C   3C         11C   4
            //  1A        3C   10A   11C   4
            //  ----------------------------

            it("by returning Solution 1", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------------
                //  1B                         1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'B',
                    '3'  : 'A',
                    '7'  : 'A', // Bariatric Flag.  A = Yes.
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);
                expect(recommendations[0].bariatricFlag).toEqual(true);

                //  ----------------------------
                //  1A   2B                    1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'B',
                    '3'  : 'A',
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

                //  ----------------------------
                //  1A        10C              1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'B',
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

                //  ----------------------------
                //  1A   2C         3B         1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'B',
                    '10' : 'C',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

                //  ----------------------------
                //  1A   2C              11B   1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '10' : 'A',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

                //  ----------------------------
                //  1A        3B   10A         1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'B',
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

                //  ----------------------------
                //  1A             10A   11B   1
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '10' : 'A',
                    '11' : 'B'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[0].recommendation);

            });

            // ---------------------------------------------------------------------------------------------------------
            // -    Algorithm Nine - Solution 2
            // ---------------------------------------------------------------------------------------------------------
            it("by returning Solution 2 for Algorithm 9", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------------
                //  1A   2A        10B         2
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '10' : 'B',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[1].recommendation);
            });

            // ---------------------------------------------------------------------------------------------------------
            // -    Algorithm Nine - Solution 3
            // ---------------------------------------------------------------------------------------------------------
            it("by returning Solution 3 for Algorithm 9", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------------
                //  1A   2C   3A         11A   3
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[2].recommendation);

                //  ----------------------------
                //  1A        3A   10A   11A   3
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'A',
                    '3'  : 'A',
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[2].recommendation);

            });

            // ---------------------------------------------------------------------------------------------------------
            // -    Algorithm Nine - Solution 4
            // ---------------------------------------------------------------------------------------------------------
            it("by returning Solution 4 for Algorithm 9", function () {
                var assessment,
                    algorithm,
                    recommendations;
                algorithm = ConstantsMock.algorithms;

                //  ----------------------------
                //  1A   2C   3C         11C   4
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'C',
                    '10' : 'A',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[3].recommendation);

                //  ----------------------------
                //  1A        3C   10A   11C   4
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'C',
                    '10' : 'A',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[3].recommendation);

                //  ----------------------------
                //  1A   2C   3A   10A   11C   4
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'A',
                    '10' : 'A',
                    '11' : 'C'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[3].recommendation);

                //  ----------------------------
                //  1A   2C   3C   10A   11A   4
                //  ----------------------------
                assessment = {
                    '1'  : 'A',
                    '2'  : 'C',
                    '3'  : 'C',
                    '10' : 'A',
                    '11' : 'A'
                };
                recommendations = RecommendationEngine.recommend(assessment, algorithm);
                expect(recommendations[0].recommendation.$$unwrapTrustedValue()).toEqual(Constants.algorithms[8].solution[3].recommendation);
            });
        });
    });
});