/*jslint node: true, unparam: true */
/*global define */

define(['angular', 'app'], function (angular, app) {
    "use strict";
    app.service('RecommendationEngine', function ($window, Constants, $log, $sce) {
        // Service is used to track the Questions.
        // Functions
        //      - getAlgorithmOptions
        //      - getSelectedAlgorithms
        //      - saveSelectedAlgorithms
        //      - clearSelectedAlgorithms
        //      - recommend
        // Flow:


        // -------------------------------------------------------------------------------------------------------------
        // -   Init
        // -------------------------------------------------------------------------------------------------------------
        var RecommendationEngine = {},
            algorithmOptionsFull = Constants.algorithms,
            selectedAlgorithms = angular.fromJson($window.localStorage.selectedAlgorithms) || [];

        // Returns a full Algorithm List, from Constants.
        RecommendationEngine.getAlgorithmOptions = function () {
            return angular.copy(algorithmOptionsFull);
        };

        RecommendationEngine.getSelectedAlgorithms = function () {
            return angular.copy(selectedAlgorithms);
        };

        RecommendationEngine.saveSelectedAlgorithms = function (algorithmsSelected) {
            selectedAlgorithms = angular.copy(algorithmsSelected);
            delete $window.localStorage.selectedAlgorithms;
            $window.localStorage.selectedAlgorithms = angular.toJson(algorithmsSelected);
        };

        RecommendationEngine.clearSelectedAlgorithms = function () {
            delete $window.localStorage.selectedAlgorithms;
            selectedAlgorithms = [];
        };

        RecommendationEngine.getChosenAlgorithms = function () {
            return angular.copy(angular.fromJson($window.localStorage.chosenAlgorithms) || []);
        };

        RecommendationEngine.saveChosenAlgorithms = function (algorithmsChosen) {
            delete $window.localStorage.chosenAlgorithms;
            $window.localStorage.chosenAlgorithms = angular.toJson(algorithmsChosen);
        };

        RecommendationEngine.clearChosenAlgorithms = function () {
            delete $window.localStorage.chosenAlgorithms;
        };


        // -------------------------------------------------------------------------------------------------------------
        // -    Recommend Function
        // -------------------------------------------------------------------------------------------------------------
        // Creates a list of the recommendations based on the answers passed in
        // Algorithms:  passed in selected values.
        // assessment:  Full question list with answers.
        RecommendationEngine.recommend = function (assessment, algorithms) {
            // Init
            var recommendations,
                generateTestValue,
                recommendSolution;

            // assessment === Full question list with answers.
            // questions === ???
            generateTestValue = function (assessment, questions) {
                var testValue = '';

                angular.forEach(questions, function (num) {
                    if (angular.isArray(assessment[num])) {
                        testValue += (num + assessment[num].join(''));
                    } else {
                        testValue += (num + assessment[num]);
                    }
                });

                return testValue;
            };

            // Loop through the giant object of the stored chosen algorithms?
            // That's what I think.
            // assessment: Object {1: "A", 2: "A", 3: "A", 4: "A", 5: "A", 6: "A", 7: "A", 8: "A", 9: "A", 10: "A", 11: "A", 12: null, 13: "A", 14: "A", 15: "A", 16: "A", 17: "A"}
            // algorithms: Object of Objects
            recommendSolution = function (assessment, algorithms) {
                // Init
                var solutions = [];

                // Start the loop at the top level of chosen algorithms
                angular.forEach(algorithms, function (algorithm) {
                    // Have to loop again; since there can be many specific algorithm rules within one algorithm.
                    // algorithm.algorithm
                    var bariatricFlag = false,
                        foundSolution = false,
                        fallGateFlag = false;
                    angular.forEach(algorithm.algorithm, function (specificAlgorithm) {
                        // Init
                        var testValue = generateTestValue(assessment, specificAlgorithm.id),
                            pattern = new RegExp(specificAlgorithm.answerCriteria);

                        //  $log.info('recommendSolution: specific Algorithm level');
                        //  $log.info(JSON.stringify(specificAlgorithm.answerCriteria));
                        //  $log.info('THE TEST VALUE IS: ' + testValue);
                        //  $log.info(pattern);
                        //  $log.info('THE ANSWER TEST IS: ' + pattern.test(testValue));

                        // Setting Bariatric Flag
                        if (testValue === '7A') {
                            bariatricFlag = true;
                        } else if (testValue === '8C' && algorithm.id === 8) {
                            fallGateFlag = true;
                        } else if (pattern.test(testValue) && foundSolution === false) {
                            // We only push a solution if it matches up (the answerCriteria to the assessment Question string.)
                            // Only one recommendation should hit.
                            foundSolution = true;
                            solutions.push({
                                id: algorithm.id,
                                name: algorithm.name,
                                bariatricFlag: bariatricFlag,
                                fallGateFlag: fallGateFlag,
                                flowChart: algorithm.flowChart,
                                notes: $sce.trustAsHtml(algorithm.notes),
                                // equipment: $sce.trustAsHtml(algorithm.solution[specificAlgorithm.recommendation].equipment),
                                equipment: algorithm.solution[specificAlgorithm.recommendation].equipment,
                                recommendation: $sce.trustAsHtml(algorithm.solution[specificAlgorithm.recommendation].recommendation)
                            });
                            // $log.info('answer criteria matched test value! ' + foundSolution);
                        }
                    });
                });

                return solutions;
            };

            $log.info('ASSESSMENT FOLLOWS: ---------------------------');
            //  $log.info(JSON.stringify(assessment));
            //  $log.info(algorithms);
            recommendations = recommendSolution(assessment, algorithms);

            return recommendations;
        };

        return RecommendationEngine;
    });
});