/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'RecommendationEngine',
    'text!/base/app/src/modules/cpaa/core/constants/cpaaQuestions.json',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_recommendation_2.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-1/algorithm_1_equipment_0.html',

    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_notes.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_recommendation_0.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_recommendation_1.html',
    'text!/base/app/src/modules/cpaa/core/constants/algorithm-2/algorithm_2_equipment_0.html'
], function (RecommendationEngine,
             incomingCpaaQuestions,
             incomingCpaaAlgorithm_1_notes,
             incomingCpaaAlgorithm_1_recommendation_0,
             incomingCpaaAlgorithm_1_recommendation_1,
             incomingCpaaAlgorithm_1_recommendation_2,
             incomingCpaaAlgorithm_1_equipment_0,

             incomingCpaaAlgorithm_2_notes,
             incomingCpaaAlgorithm_2_recommendation_0,
             incomingCpaaAlgorithm_2_recommendation_1,
             incomingCpaaAlgorithm_2_equipment_0) {
    'use strict';

    describe("The RecommendationEngine", function () {
        var RecommendationEngine,
            Constants,
            ConstantsMock,
            selectedAlgorithms,
            $window,
            store = {},
            $sce,
            cpaaQuestions = JSON.parse(incomingCpaaQuestions),
            cpaaAlgorithm_1_notes               = incomingCpaaAlgorithm_1_notes.toString(),
            cpaaAlgorithm_1_recommendation_0    = incomingCpaaAlgorithm_1_recommendation_0.toString(),
            cpaaAlgorithm_1_recommendation_1    = incomingCpaaAlgorithm_1_recommendation_1.toString(),
            cpaaAlgorithm_1_recommendation_2    = incomingCpaaAlgorithm_1_recommendation_2.toString(),
            cpaaAlgorithm_1_equipment_0         = incomingCpaaAlgorithm_1_equipment_0.toString(),

            cpaaAlgorithm_2_notes               = incomingCpaaAlgorithm_2_notes.toString(),
            cpaaAlgorithm_2_recommendation_0    = incomingCpaaAlgorithm_2_recommendation_0.toString(),
            cpaaAlgorithm_2_recommendation_1    = incomingCpaaAlgorithm_2_recommendation_1.toString(),
            cpaaAlgorithm_2_equipment_0         = incomingCpaaAlgorithm_2_equipment_0.toString();

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            assessment: {
                '1': 'B',
                '2': 'C',
                '3': 'C',
                '4': 'B',
                '5': 'B',
                '6': 'B',
                '7': 'A',
                '8': 'C',
                '9': 'C',
                '10': 'C',
                '11': 'B',
                '12': null,
                '13': 'C',
                '14': 'B',
                '15': 'A',
                '16': 'C',
                '17': 'B'
            },
            questions: cpaaQuestions,
            algorithms: [
                {
                    id: 1,
                    name: 'Transfer To/From Seated Positions: Bed to Chair, Chair to Chair, Chair to Exam Table',
                    bariatricFlag: false,
                    flowChart: 'path/to/chart/image.png',
                    notes: cpaaAlgorithm_1_notes,
                    questions: [1, 2, 3, 7, 10, 11],
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '7A',
                            recommendation: 0
                        },
                        {
                            id: [1, 7],
                            answerCriteria: '^1B.*',
                            recommendation: 0
                        }, {
                            id: [1, 2, 7, 10],
                            answerCriteria: '^1A(2B.*.*|.*.*10C)',
                            recommendation: 0
                        }, {
                            id: [1, 2, 7, 10],
                            answerCriteria: '^1A2A.*10B',
                            recommendation: 1
                        }, {
                            id: [1, 2, 3, 7, 10, 11],
                            answerCriteria: '^1A2C(3A|3C).*10A(11A|11C)',
                            recommendation: 2
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_1_recommendation_0
                        }, {
                            recommendation: cpaaAlgorithm_1_recommendation_1
                        }, {
                            recommendation: cpaaAlgorithm_1_recommendation_2
                        }
                    ]
                }, {
                    id: 2,
                    name: 'Lateral Transfer to/from Supine Positions: Bed, Stretcher, Trolley, Procedure Table',
                    bariatricFlag: false,
                    flowChart: 'path/to/chart/image.png',
                    notes: cpaaAlgorithm_2_notes,
                    questions: [1, 4, 7],
                    algorithm: [
                        {
                            id: [7],
                            answerCriteria: '^7A',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1B.*',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1A4B',
                            recommendation: 0
                        },
                        {
                            id: [1, 4],
                            answerCriteria: '^1A4A',
                            recommendation: 1
                        }
                    ],
                    solution: [
                        {
                            recommendation: cpaaAlgorithm_2_recommendation_0,
                            equipment: cpaaAlgorithm_2_equipment_0              // All recommendation equipment is same.
                        }, {
                            recommendation: cpaaAlgorithm_2_recommendation_1,
                            equipment: cpaaAlgorithm_2_equipment_0              // All recommendation equipment is same.
                        }
                    ]
                }
            ]
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            // Sets the selectedAlgorithms to a multi-selected item of 2.
            localStorage.setItem('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));

            module(function ($provide) {
                $provide.constant('Constants', ConstantsMock);
                $provide.constant('selectedAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
                $provide.constant('chosenAlgorithms', angular.toJson(ConstantsMock.algorithms[0]));
            });


            inject(function (_RecommendationEngine_, _$window_, _$sce_) {
                RecommendationEngine = _RecommendationEngine_;
                $window = _$window_;
                $sce = _$sce_;
            });
        });

        describe("should test the load and saving", function () {
            it("getAlgorithmOptions", function () {
                expect(RecommendationEngine.getAlgorithmOptions).toBeDefined();
                RecommendationEngine.getAlgorithmOptions();
                expect(RecommendationEngine.getAlgorithmOptions()).toEqual(ConstantsMock.algorithms);
            });
            it("getSelectedAlgorithms", function () {
                var selectedAlgorithms = JSON.parse(localStorage.getItem('selectedAlgorithms'));
                RecommendationEngine.getSelectedAlgorithms();
                expect(RecommendationEngine.getSelectedAlgorithms).toBeDefined();
                expect(RecommendationEngine.getSelectedAlgorithms()).toEqual(ConstantsMock.algorithms[0]);
            });
            it("saveSelectedAlgorithms", function () {
                var selectedAlgorithms = JSON.parse(localStorage.getItem('selectedAlgorithms'));

                expect(RecommendationEngine.saveSelectedAlgorithms).toBeDefined();

                RecommendationEngine.saveSelectedAlgorithms(angular.fromJson(ConstantsMock.algorithms[0]));
                expect(selectedAlgorithms).toEqual(ConstantsMock.algorithms[0]);
            });
        });
    });
});