/*jslint node: true, unparam: true */
/*globals define */

define([
    'app',
    'Constants',
    'RecommendationEngine',
    'CPAAPrimaryWrapperController',
    'AssessmentController',
    'AlgorithmsController',
    'RecommendationsController',
    'AssessmentInstanceBuilder',
    'AssessmentService'
], function (app) {
    "use strict";

    app.config(function ($stateProvider) {
        $stateProvider
            .state('main.two-panel.secondary-navigation.cpaa', {
                url: '/cpaa',
                appNavIndex: 2,
                data: {
                    panel: "secondary",
                    appletName: "Comprehensive Patient Assessment & Algorithms",
                    headerTitle: "Safe Patient Handling",
                    baseStateName: 'main.two-panel.secondary-navigation.cpaa',
                    initialState: "assessment",
                    paramSections: [
                        {
                            paramValue: "multi-tasks",
                            section: "multi-tasks",
                            buttonClass: "option1",
                            sectionName: "Conduct an assessment for multiple tasks",
                            sectionIndex: 1,
                            algorithms: [],
                            nowDoAssessment: 'no'
                        },
                        {
                            paramValue: "single-task",
                            section: "single-task",
                            buttonClass: "option2",
                            sectionName: "Select a Single Task",
                            sectionIndex: 2,
                            algorithms: [],
                            nowDoAssessment: 'no'
                        }
                    ]
                },
                views: {
                    'primary@main.two-panel': {
                        templateUrl: 'src/modules/cpaa/components/primary-wrapper/cpaa-primary-wrapper_template.html',
                        controller: 'CPAAPrimaryWrapperController'
                    },
                    'primary-content': {
                        templateUrl: 'src/modules/cpaa/pages/assessment/assessment_template.html',
                        controller: 'AssessmentController'
                    }
                }
            })
            // Assessment is the Question List
            .state('main.two-panel.secondary-navigation.cpaa.assessment', {
                url: '/{section:single-task|multi-tasks}',
                data: {
                    panel: "primary",
                    backState: "^"
                },
                views: {
                    'primary-content': {
                        templateUrl: 'src/modules/cpaa/pages/assessment/assessment_template.html',
                        controller: 'AssessmentController'
                    }
                },
                params: {
                    algorithms: [],
                    section: '',
                    nowDoAssessment: 'no'
                },
                onEnter: function ($state, $stateParams) {
                    // Section should always be single-tasks
                    // The algorithms should only be populated after the first single-task
                    if ($stateParams.section === 'single-task' && $stateParams.nowDoAssessment === 'no') {
                        $state.go('main.two-panel.secondary-navigation.cpaa.algorithms', $stateParams, {reload: true});
                    }
                }
            })
            // Selection is the Algorithm List
            .state('main.two-panel.secondary-navigation.cpaa.algorithms', {
                url: '/{section:single-task|multi-tasks}',
                data: {
                    panel: "primary",
                    backState: "^.assessment"
                },
                params: {
                    algorithms: [],
                    section: '',
                    nowDoAssessment: 'no'
                },
                views: {
                    'primary-content': {
                        templateUrl: 'src/modules/cpaa/pages/algorithms/algorithms_template.html',
                        controller: 'AlgorithmsController'
                    }
                }
            })
            // Recommendations is the Solution Flowchart List
            .state('main.two-panel.secondary-navigation.cpaa.recommendations', {
                url: '/{section:single-task|multi-tasks}',
                data: {
                    panel: "primary",
                    backState: "^.algorithms"
                },
                params: {
                    algorithms: [],
                    section: '',
                    nowDoAssessment: 'no'
                },
                views: {
                    'primary-content': {
                        templateUrl: 'src/modules/cpaa/pages/recommendations/recommendations_template.html',
                        controller: 'RecommendationsController'
                    }
                }
            });
    });
});