/*jslint node: true */
/*global define, controller */

define(['angular', 'app'], function (angular, app) {
    'use strict';

    app.controller('AlgorithmsController', [
        '$scope', '$state', '$stateParams', '$filter', 'Constants', 'AssessmentInstanceBuilder', 'RecommendationEngine', '$log', '$window',
        function ($scope, $state, $stateParams, $filter, Constants, AssessmentInstanceBuilder, RecommendationEngine, $log, $window) {
            // Controller is used to show all the Algorithms.
            // Can be first if single-task, or second if multi-task
            // Flow:
            //      - Get AnsweredQuestions IF any (to check for Fall Algorithm)
            //      - Get All Algorithms
            //      - Get ChosenAlgorithms IF any
            //      - Check to see if need to display Fall Algorithm.
            //      - Display list, with option(s) selected (based on single/multi).
            //      - On Save; save/update selected Algorithms to localStorage
            //      - On Save go to Recommendations (if multi) or Assessment (if single)
            $log.info('AlgorithmsController; params follow');
            //  $log.info($stateParams);
            //  $log.info($stateParams.section);

            // -------------------------------------------------------------------------------------------------------------
            // -    Init Section, setting default values.
            // -------------------------------------------------------------------------------------------------------------
            var service = AssessmentInstanceBuilder.getInstance(),
                questions,
                fallGateQuestion,
                fallGateAnswer,
                fallAlgorithmIndex;

            // Populating $scope JSON objects
            // chosenAlgorithms = the selectedAlgorithms in Object form.
            // algorithmList:   Just a JS Object of all the Algorithms, from Constants.
            $scope.chosenAlgorithms = RecommendationEngine.getChosenAlgorithms(); // does nothing - is broken.
            $scope.selectedAlorithms = {};
            $scope.algorithmList = Constants.algorithms;
            $scope.disabledAlgorithms = [];

            // Simple function to check is the assessment if filled out in localStorage.
            $scope.checkForAssessment = function () {
                return $window.localStorage.assessment ? true : false;
            };

            // Cheating the unit test here, calling a function (which i can mock) instead of just checking for localStorage.assessment
            if ($scope.checkForAssessment()) {
                questions = service.returnAllQuestionsWithAnswers();
                fallGateQuestion = _.find(questions, {id: 12});
                if (!!fallGateQuestion.choice) {
                    fallGateAnswer = fallGateQuestion.choice[5] || null;
                }
                fallAlgorithmIndex = _.findIndex(Constants.algorithms, {id: 8});

                if (fallGateAnswer) {
                    $scope.disabledAlgorithms[fallAlgorithmIndex] = !!fallGateAnswer;
                }
            }


            // ---------------------------------------------------------------------------------------------------------
            // -    Module Functions
            // ---------------------------------------------------------------------------------------------------------

            // Setting Single or Multi Choice.
            // passed in as a $stateParam
            if ($stateParams.section === 'single-task') {
                $scope.singleMultiChoice = 'radio';
                $scope.nextStepText = 'Next: Complete assessment questions for the selected task';
            } else {
                $scope.singleMultiChoice = 'checkbox';
                $scope.nextStepText = 'Get Recommendations';
            }

            // Single-task function only.
            // Because of the way the radio buttons are generated; they won't clear out after you select one.
            // so it creates an array
            //      [ null, null, 3, null].
            // but if you choose option 4, it becomes
            //      [ null, null, 3, 4 ]
            $scope.chooseOnlyThisAlgorithm = function (chosenAlgorithm) {
                angular.forEach($scope.chosenAlgorithms, function (value, key) {
                    $log.info('value:' + value + ' and ' + chosenAlgorithm);
                    if ($filter('number')(value) !== $filter('number')(chosenAlgorithm)) {
                        value = null;
                        $scope.chosenAlgorithms[key] = null;
                    }
                });
            };

            // Check for a selectedAlgorithm
            $scope.checkForSelectedAlgorithm = function (chosenAlgorithms) {
                var isAlgorithmSelected = true;
                //  $log.info('AlgorithmsController: checkForSelectedAlgorithm');
                //  $log.info($scope.chosenAlgorithms);
                if ($scope.chosenAlgorithms.length > 0) {
                    angular.forEach(chosenAlgorithms, function (value) {
                        if (value !== 'false' && value !== undefined && value !== false) {
                            $log.info(value);
                            isAlgorithmSelected = false;
                        }
                    });
                }
                return isAlgorithmSelected;
            };

            // Function used to open the flowchart in a new tab.
            // url is the Constant.algorithm.flowchart variable, currently the full path.
            $scope.viewFlowChart = function (url) {
                $window.open(url, '_blank');
            };

            // TODO: Poor named function; this is to move onto the Recommendations.
            $scope.next = function () {
                $log.info('AlgorithmsController: next function called:');

                // Creating empty selected AlgorithmKeys.
                // seems pointless, have chosenAlgorithms json object.
                var selectedAlgorithmKeys = [];

                // Loops over the ng-model and pushes the json object into stateparams.
                angular.forEach($scope.chosenAlgorithms, function (value, index) {
                    if (value) {
                        selectedAlgorithmKeys.push($scope.algorithmList[index]);
                        //  $log.info('value then index: ' + value + ', ' + index);
                        //  $log.info($scope.algorithmList[index]);
                        //  $log.info('added to keys -----');
                        //  $log.info(selectedAlgorithmKeys);
                    }
                });

                //  $log.info('set the stateParams with the selected Keys:');
                //  $log.info(selectedAlgorithmKeys);
                //  $log.info($scope.chosenAlgorithms);
                //  $log.info($stateParams);
                $stateParams.algorithms = selectedAlgorithmKeys;

                RecommendationEngine.saveSelectedAlgorithms($stateParams.algorithms);
                RecommendationEngine.saveChosenAlgorithms($scope.chosenAlgorithms);
                //  $state.go('contact.detail') will go to the 'contact.detail' state
                //  $state.go('^') will go to a parent state.
                //  $state.go('^.sibling') will go to a sibling state.
                //  $state.go('.child.grandchild') will go to a grandchild state.
                //
                //  Thus this is going to a sibling state of recommendations.
                //  You can see the state in the main module.js
                //  $log.info('NOW GO STATE POWER GO!!!!!!!!');

                // This seems wrong
                if ($stateParams.section === 'single-task') {
                    $state.go('^.assessment', {
                        section: $stateParams.section,
                        algorithms: $stateParams.algorithms,
                        nowDoAssessment: 'yes'
                    });
                } else {
                    $state.go('^.recommendations', {
                        section: $stateParams.section,
                        algorithms: $stateParams.algorithms,
                        nowDoAssessment: 'no'
                    });
                }
            };
        }]
    );
});
