/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define(['AssessmentController'], function () {
    'use strict';

    describe("The AssessmentController controller", function () {
        var createController,
            scope,
            httpBackend,
            stateParams,
            AssessmentInstanceBuilderMock,
            stateParamsMock,
            window;


        // Will have a unit test just on teh Assessment Service.
        // This just mocks up the functions so it returns what we want.
        AssessmentInstanceBuilderMock = {
            getInstance: function () {
                return {
                    returnAllQuestions: function () {
                        return [
                            {
                                id: 7,
                                question: 'Is the patient greater than 300 lb or 136 kg?',
                                options: ['Yes', 'No']
                            },
                            {
                                id: 2,
                                question: 'Weight Bearing Capability',
                                options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing']
                            },
                            {

                                id: 10,
                                question: 'Balance',
                                options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance']
                            },
                            {
                                id: 3,
                                question: 'Upper Extremity Strength',
                                options: ['Full Strength in both upper extremities',
                                    'No Strength in either upper extremity',
                                    'Strength in one upper extremity']
                            }
                        ];
                    },
                    returnAllQuestionsWithAnswers: function () {
                        return [
                            {
                                id: 7,
                                question: 'Is the patient greater than 300 lb or 136 kg?',
                                options: ['Yes', 'No']
                            },
                            {
                                id: 2,
                                question: 'Weight Bearing Capability',
                                options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing']
                            },
                            {

                                id: 10,
                                question: 'Balance',
                                options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance']
                            },
                            {
                                id: 3,
                                question: 'Upper Extremity Strength',
                                options: ['Full Strength in both upper extremities',
                                    'No Strength in either upper extremity',
                                    'Strength in one upper extremity']
                            }
                        ];
                    },
                    returnSpecificAlgorithmQuestions: function () {
                        return [
                            {
                                id: 7,
                                question: 'Is the patient greater than 300 lb or 136 kg?',
                                options: ['Yes', 'No']
                            }
                        ];
                    }
                };
            }
        };
        stateParamsMock = {
            algorithms: [],
            section: "single-task",
            nowDoAssessment: 'no'
        };

        beforeEach(function () {
            module('angularTemplateApp');

            module(function ($provide) {
                $provide.value('AssessmentInstanceBuilder', AssessmentInstanceBuilderMock);
            });

            inject(function ($controller, $rootScope, _$httpBackend_, _$state_, _$window_) {
                scope = $rootScope.$new();
                httpBackend = _$httpBackend_;
                stateParams = stateParamsMock;
                window = _$window_;

                createController = function () {
                    return $controller('AssessmentController', {
                        $scope: scope,
                        AssessmentInstanceBuilder: AssessmentInstanceBuilderMock,
                        $state: _$state_,
                        $stateParams: stateParamsMock
                    });
                };
            });
        });

        describe("should load the questions", function () {
            it("for a single-task", function () {
                stateParamsMock = {
                    algorithms: [],
                    section: "single-task",
                    nowDoAssessment: 'yes'
                };
                var controller = createController();

                expect(scope.questions[0].id).toEqual(7);
                expect(scope.questions.length).toEqual(1);
            });
            it("for a multi-task", function () {
                stateParamsMock = {
                    algorithms: [],
                    section: "multi-tasks",
                    nowDoAssessment: 'no'
                };
                var controller = createController();

                expect(scope.questions[0].id).toEqual(7);
                expect(scope.questions.length).toEqual(4);
            });
        });
    });
});
