/*jslint node: true */
/*globals define */

define(['angular', 'app'], function (angular, app) {
    'use strict';

    app.controller('RecommendationsController', [
        '$scope', '$state', '$stateParams', '$window', 'AssessmentInstanceBuilder', 'RecommendationEngine', '$log',
        function ($scope, $state, $stateParams, $window, AssessmentInstanceBuilder, RecommendationEngine, $log) {
            // RecommendationsController is used to show flowChart/Solution
            // Flow:
            //      - Get Answered Algorithms
            //      - Get Answered Questions
            //      - Send those two items to RecommendationEngine
            $log.info('RecommendationsController:');

            $scope.formatType = ($stateParams.section === 'multi-tasks') ? 'multi' : 'single';

            // -------------------------------------------------------------------------------------------------------------
            // -    Init Section, setting default values.
            // -------------------------------------------------------------------------------------------------------------
            var service = AssessmentInstanceBuilder.getInstance(),
                algorithms = RecommendationEngine.getSelectedAlgorithms();


            //  algorithms:  either the locallyStored collection (full Algorithm list, with either partial or full answers in there) or a copy of the Constants if nothing stored locally
            //  service.getAnswerSheet() returns Object {1: "A", 2: "A", 3: "A", 4: "A", 5: "A", 6: "A", 7: "A", 8: "A", 9: "A", 10: "A", 11: "A", 12: null, 13: "A", 14: "A", 15: "A", 16: "A", 17: "A"}
            $scope.recommendations = RecommendationEngine.recommend(service.getAnswerSheet(), algorithms);

            $log.info('RecommendationEngine.recommend - recommendations, algorithms follow:');
            $log.info($scope.recommendations);
            $log.info(algorithms);

            RecommendationEngine.clearSelectedAlgorithms();
            RecommendationEngine.clearChosenAlgorithms();

        }
    ]);
});
