/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define(['RecommendationsController'], function () {
    'use strict';

    describe("The RecommendationsController controller", function () {
        var controller,
            createController,
            scope,
            stateParams,
            AssessmentInstanceBuilderMock,
            RecommendationEngineMock,
            ConstantsMock,
            stateParamsMock,
            windowMock,
            window;

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        ConstantsMock = {
            questions: [
                {
                    id: 7,
                    question: 'Is the patient greater than 300 lb or 136 kg?',
                    options: ['Yes', 'No']
                },
                {
                    id: 2,
                    question: 'Weight Bearing Capability',
                    options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing']
                },
                {

                    id: 10,
                    question: 'Balance',
                    options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance']
                }
            ],
            answerSheet: [
                {
                    id: 7,
                    question: 'Is the patient greater than 300 lb or 136 kg?',
                    options: ['Yes', 'No'],
                    selection: ['Yes']
                },
                {
                    id: 2,
                    question: 'Weight Bearing Capability',
                    options: ['Fully Weight Bearing', 'Non-Weight Bearing', 'Partial Weight Bearing'],
                    selection: ['Non-Weight Bearing']
                },
                {

                    id: 10,
                    question: 'Balance',
                    options: ['Ability to maintain sitting balance at edge of bed/chair', 'Ability to maintain standing balance', 'No sitting or standing balance'],
                    selection: ['Ability to maintain standing balance']
                }
            ],
            algorithms: [{
                id: 2,
                name: 'Lateral Transfer to/from Supine Positions: Bed, Stretcher, Trolley, Procedure Table',
                flowChart: 'path/to/chart/image.png',
                notes: 'yes test common recommendation',
                questions: [1, 4],
                algorithm: [
                    {
                        id: [1],
                        answerCriteria: '/^1B$/',
                        recommendation: 0
                    }, {
                        id: [1, 4],
                        answerCriteria: '/^1A4B$/',
                        recommendation: 0
                    }, {
                        id: [1, 4],
                        answerCriteria: '/^1A4A$/',
                        recommendation: 1
                    }
                ],
                solution: [
                    {
                        recommendation: 'A Recommendation'
                    }, {
                        recommendation: 'B Recommendation'
                    }
                ]
            }]
        };
        AssessmentInstanceBuilderMock = {
            getInstance: function () {
                return {
                    returnAllQuestions: function () {
                        return ConstantsMock.questions;
                    },
                    returnSpecificAlgorithmQuestions: function () {
                        return ConstantsMock.algorithms;
                    },
                    getAnswerSheet: function () {
                        return ConstantsMock.answerSheet;
                    }
                };
            }
        };
        RecommendationEngineMock = {
            recommend: function () {
                return ConstantsMock.algorithms[0].solution[0].recommendation;
            },
            getAlgorithmOptions: function () {
                return ConstantsMock.questions;
            },
            getSelectedAlgorithms: function () {
                return ConstantsMock.questions[0];
            },
            saveSelectedAlgorithms: function (algorithmSaved) {
                return algorithmSaved;
            },
            clearSelectedAlgorithms: function () {
                return true;
            },
            getChosenAlgorithms: function () {
                return ConstantsMock.questions[0];
            },
            saveChosenAlgorithms: function (algorithmSaved) {
                return algorithmSaved;
            },
            clearChosenAlgorithms: function () {
                return true;
            },
        };
        stateParamsMock = {
            algorithms: [angular.fromJson(ConstantsMock.algorithms)],
            section: "single-task",
            nowDoAssessment: 'no'
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            module(function ($provide) {
                $provide.value('AssessmentInstanceBuilder', AssessmentInstanceBuilderMock);
                $provide.value('RecommendationEngine', RecommendationEngineMock);
                $provide.value('Constants', ConstantsMock);
                $provide.service('$window', function () {
                    windowMock = jasmine.createSpyObj('$window', ['open']);
                    return windowMock;
                });
            });

            inject(function ($controller, $rootScope) {
                scope = $rootScope.$new();
                stateParams = stateParamsMock;

                createController = function () {
                    return $controller('RecommendationsController', {
                        $scope: scope,
                        AssessmentInstanceBuilder: AssessmentInstanceBuilderMock,
                        RecommendationEngine: RecommendationEngineMock,
                        Constants: ConstantsMock,
                        $stateParams: stateParamsMock
                    });
                };
                // this way if I need to re-instantiate the controller within and 'it' I could by calling this line.
                controller = createController();
            });
        });

        describe("should load", function () {
            it("the controller", function () {
                expect(scope.recommendations).toEqual('A Recommendation');
            });
        });

    });
});
