/*jslint node: true, unparam: true */
/*global define, controller */


define(['angular', 'app'], function (angular, app) {
    'use strict';
    app.controller('SASTPrimaryWrapperController', [
        '$scope', '$rootScope', '$state', '$stateParams', '$filter', '$modal', '$log', 'modalService', 'focusService', 'SASTAssessmentService', '$timeout',
        function ($scope, $rootScope, $state, $stateParams, $filter, $modal, $log, modalService, focusService, SASTAssessmentService, $timeout) {
            // Controller is used to to wrap the app..
            // Flow:
            //      -
            // Module Functions:
            $log.info('SASTPrimaryWrapperController');

            // ---------------------------------------------------------------------------------------------------------
            // -    Module Functions
            // ---------------------------------------------------------------------------------------------------------
            $scope.infoButton = function () {
                var modalContent = {
                    closeButtonText: 'OK',
                    headerText: 'Sample Assessment Process',
                    bodyContentUrl: 'src/ui-components/modals/sast-info/sast-info_template.html'
                };

                modalService.showModal({}, modalContent).then(function () {
                    $log.info('to stop empty block');
                }, function () {
                    focusService.focusElement(angular.element('button[title="Info"]'));
                });
            };

            // ---------------------------------------------------------------------------------------------------------
            // -    Clear Assessment
            // ---------------------------------------------------------------------------------------------------------
            $scope.clearAssessment = function () {
                var modalContent = {
                    actionButtonText: 'OK',
                    closeButtonText: 'Cancel',
                    headerText: 'Confirm clear data',
                    bodyText: 'Confirming this action will clear all assessment question responses, selected tasks, and recommendations.',
                };

                modalService.showModal({}, modalContent).then(
                    //  What to do when OK button is tapped
                    function () {
                        var service = new SASTAssessmentService();

                        service.delete();
                        $state.go('main.two-panel.secondary-navigation.sast');
                    },
                    //  What to do when Cancel button is tapped
                    function () {
                        // When modal is closed refocus on the button that opened it.
                        focusService.focusElement(angular.element('button[title="Clear Assessment"]'));
                    }
                );
            };

            // ---------------------------------------------------------------------------------------------------------
            // -    Listeners: Update Score
            // ---------------------------------------------------------------------------------------------------------
            $rootScope.$on('UpdateScore', function (event, score) {
                var realScore = parseInt(score, 10);
                $timeout(function () {
                    if (realScore >= 4) {
                        $scope.score = 4;
                    } else {
                        $scope.score = score;
                    }
                }, 50);
            });

            // ---------------------------------------------------------------------------------------------------------
            // -    Change Events
            // ---------------------------------------------------------------------------------------------------------
            $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams, $stateParams) {
                $stateParams = toState;

                //  $log.info(toState);
                //  $log.info(toParams);
                //  So sometimes we have $stateParams coming in; if so we want to keep them.
                //  We're also looking specifically at the single vs. multi task; so we can assign the correct sectionName.
                //  $scope.sectionName = toState.data.primaryHeader || $filter('filter')(toState.data.paramSections, {paramValue: $stateParams.section}, true)[0].sectionName;
                $stateParams.section = toParams.section;
                $scope.sectionName = toState.data.primaryHeader || toParams.sectionName;
                $scope.score = toParams.score;
                $scope.headerButton = toState.data.headerButton;
            });
        }
    ]);
});