/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'SASTRecommendationEngine',
    'SASTConstants'
], function (SASTRecommendationEngine, SASTConstants) {
    'use strict';

    describe("Algorithm One", function () {
        var RecommendationEngine,
            Constants,
            mockAssessment,
            algorithm = {};

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        mockAssessment = [
            {
                "id": 1,
                "question": "Patient’s Level of Assistance:",
                "options": [
                    "No Help Needed – The patient is independent, standby for safety only, needs no staff assist ",
                    "The patient needs minimal to moderate help, light touch, verbal cues – Staff is required to lift <35 #",
                    "Dependent—Patient requires staff to lift > 35# or the patient has unpredictable strength/high risk for fall during the reposition. In this case assistive devices need to be used for safe moves."
                ],
                "score": [0, 1, 2]
            }, {
                "id": 2,
                "question": "Lower extremity ability to bear weight",
                "options": [
                    "Bilateral Weight Bearing",
                    "Bilateral but the legs are weak",
                    "Unilateral weight bearing",
                    "None (Inconsistent = none)"
                ],
                "score": [0, 1, 1, 2]
            }, {
                "id": 3,
                "question": "Upper Extremity Strength and Hand grasp",
                "options": [
                    "Bilateral strength and hand grasp present",
                    "Bilateral but the arms are weak",
                    "Unilateral Upper extremity Strength/grasp",
                    "No upper body strength/ability to grasp"
                ],
                "score": [0, 1, 1, 2]
            }, {
                "id": 4,
                "question": "Does the patient understand direction, cooperate, and follow simple commands?",
                "options": [
                    "Yes",
                    "No (Highly Unpredictable, behavior varies. Obtain new score prior to every move)",
                    "Combative during hands on care/transfers"
                ],
                "score": [0, 2, 3]
            }, {
                "id": 5,
                "question": "Height and Weight Score",
                "options": [
                    "under 200 pounds",
                    "over 200 pounds or 6 feet tall and currently has mobility issues (If the patient has a BMI over 50 or weighs over 300 pounds consider using the  Comprehensive Patient ….)"
                ],
                "score": [0, 1]
            }, {
                "id": 6,
                "question": "Check with front line staff - Does the patient have medical or mental conditions that may be likely to affect transfer/repositioning techniques such as medical instability, fractures, recent major surgery, chest pain, pressure ulcers, high risk for falls, or extremely dizzy. ",
                "options": [
                    "If there are no additional factors that complicate patient transfer",
                    "Add 1 If additional factors that make the movement/transfer more difficult per primary nurse"
                ],
                "score": [0, 1]
            }
        ];

        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            inject(function (_SASTRecommendationEngine_, _SASTConstants_) {
                RecommendationEngine = _SASTRecommendationEngine_;
                Constants = _SASTConstants_;
                algorithm.id = 4;
            });
        });

        describe("should test the score for Transfer: Bed to Chair, Chair to Toilet, Chair to Chair, Car to Chair", function () {
            it("by returning a score of 0, and getting recommendation 1", function () {

                expect(RecommendationEngine.score).toBeDefined();

                // Answer 1 - 1st option. 0 points.
                mockAssessment[0].choice = mockAssessment[0].options[0];
                // Answer 2 - 1st option. 0 points.
                mockAssessment[1].choice = mockAssessment[1].options[0];
                // Answer 3 - 1st option. 0 points.
                mockAssessment[2].choice = mockAssessment[2].options[0];
                // Answer 4 - 1st option. 0 points.
                mockAssessment[3].choice = mockAssessment[3].options[0];
                // Answer 5 - 1st option. 0 points.
                mockAssessment[4].choice = mockAssessment[4].options[0];
                // Answer 6 - 1st option. 0 points.
                mockAssessment[5].choice = mockAssessment[5].options[0];

                expect(RecommendationEngine.score(mockAssessment)).toEqual(0);
            });
            it("by returning a score of 1, and getting recommendation 2", function () {

                expect(RecommendationEngine.score).toBeDefined();

                // Answer 1 - [0, 1, 2]
                mockAssessment[0].choice = mockAssessment[0].options[1];
                // Answer 2 - [0, 1, 1, 2]
                mockAssessment[1].choice = mockAssessment[1].options[0];
                // Answer 3 - [0, 1, 1, 2]
                mockAssessment[2].choice = mockAssessment[2].options[0];
                // Answer 4 - [0, 2, 3]
                mockAssessment[3].choice = mockAssessment[3].options[0];
                // Answer 5 - [0, 1]
                mockAssessment[4].choice = mockAssessment[4].options[0];
                // Answer 6 - [0, 1]
                mockAssessment[5].choice = mockAssessment[5].options[0];

                expect(RecommendationEngine.score(mockAssessment)).toEqual(1);
            });
            it("by returning a score of 2, and getting recommendation 3", function () {

                expect(RecommendationEngine.score).toBeDefined();

                // Answer 1 - [0, 1, 2]
                mockAssessment[0].choice = mockAssessment[0].options[1];
                // Answer 2 - [0, 1, 1, 2]
                mockAssessment[1].choice = mockAssessment[1].options[2];
                // Answer 3 - [0, 1, 1, 2]
                mockAssessment[2].choice = mockAssessment[2].options[0];
                // Answer 4 - [0, 2, 3]
                mockAssessment[3].choice = mockAssessment[3].options[0];
                // Answer 5 - [0, 1]
                mockAssessment[4].choice = mockAssessment[4].options[0];
                // Answer 6 - [0, 1]
                mockAssessment[5].choice = mockAssessment[5].options[0];

                expect(RecommendationEngine.score(mockAssessment)).toEqual(2);
            });
            it("by returning a score of 3, and getting recommendation 4", function () {

                expect(RecommendationEngine.score).toBeDefined();

                // Answer 1 - [0, 1, 2]
                mockAssessment[0].choice = mockAssessment[0].options[1];
                // Answer 2 - [0, 1, 1, 2]
                mockAssessment[1].choice = mockAssessment[1].options[2];
                // Answer 3 - [0, 1, 1, 2]
                mockAssessment[2].choice = mockAssessment[2].options[0];
                // Answer 4 - [0, 2, 3]
                mockAssessment[3].choice = mockAssessment[3].options[0];
                // Answer 5 - [0, 1]
                mockAssessment[4].choice = mockAssessment[4].options[1];
                // Answer 6 - [0, 1]
                mockAssessment[5].choice = mockAssessment[5].options[0];

                expect(RecommendationEngine.score(mockAssessment)).toEqual(3);
            });
            it("by returning a score of 4, and getting recommendation 5", function () {

                expect(RecommendationEngine.score).toBeDefined();

                // Answer 1 - [0, 1, 2]
                mockAssessment[0].choice = mockAssessment[0].options[1];
                // Answer 2 - [0, 1, 1, 2]
                mockAssessment[1].choice = mockAssessment[1].options[2];
                // Answer 3 - [0, 1, 1, 2]
                mockAssessment[2].choice = mockAssessment[2].options[4];
                // Answer 4 - [0, 2, 3]
                mockAssessment[3].choice = mockAssessment[3].options[3];
                // Answer 5 - [0, 1]
                mockAssessment[4].choice = mockAssessment[4].options[1];
                // Answer 6 - [0, 1]
                mockAssessment[5].choice = mockAssessment[5].options[1];

                expect(RecommendationEngine.score(mockAssessment)).toEqual(4);
            });
        });
    });
});