/*jslint node: true, unparam: true */
/*global define */

define(['angular', 'app'], function (angular, app) {
    "use strict";
    app.service('SASTRecommendationEngine', function ($window, SASTConstants, $stateParams, $log) {
        // Service is used to score the algorithms.
        // Functions
        //      - score
        // Flow:


        // -------------------------------------------------------------------------------------------------------------
        // -   Init
        // -------------------------------------------------------------------------------------------------------------
        var RecommendationEngine = {},
            algorithm = SASTConstants.algorithms[$stateParams.sectionIndex - 1];

        $log.info('SASTRecommendationEngine');

        // -------------------------------------------------------------------------------------------------------------
        // -    Recommend Function
        // -------------------------------------------------------------------------------------------------------------
        // Creates a list of the recommendations based on the answers passed in
        // Algorithms:  passed in selected values.
        // assessment:  Full question list with answers.
        RecommendationEngine.score = function (assessment) {
            // Init
            var score = 0,
                algorithm24Cflag = false;

            //  $scope.question is array of the questions from the JSON, with the chosen value in there.
            // This will loop through the ng-model of question, and extra the total value of the form.
            angular.forEach(assessment, function (question) {
                // now matching up the selected value (full text, not number due to duplication) to get index.
                // we get index because the question.score maps out same index value.

                angular.forEach(question.options, function (option, index) {
                    // We check for choice === option or choice === true.
                    // Option means radio button.
                    // true/false means checkbox.

                    // Here we do some very specific 'hacks' for two fringe cases.
                    // ---------------------------------------------------------------------------------------------
                    // Algorithm 2 (Lateral Transfer to & From: Bed to Stretcher/Trolley)
                    // - IF the score is more than 4
                    // - IF they answered 4C
                    // - THEN show additional Recommendation Text (by passing a score of 5)
                    // ---------------------------------------------------------------------------------------------
                    if (!!algorithm) {
                        if (algorithm.id === 2) {
                            if (question.id === 4 && question.choice === option) {
                                if (index === 2) {
                                    algorithm24Cflag = true;
                                }
                            }
                        }
                    }

                    if (question.choice === option) {
                        score = score + question.score[index];
                    } else if (question.choice === true) {
                        score = score + 1;
                    }
                    // Max the score at 4.  (which gives us 5 recommendations, 0, 1, 2, 3, 4.)
                    if (score >= 4 && algorithm24Cflag === false) {
                        score = 4;
                    } else if (score >= 4 && algorithm24Cflag === true) {
                        score = 5;
                    }
                });
            });
            return score;
        };

        return RecommendationEngine;
    });
});