/*jslint node: true, unparam: true */
/*global define, controller */

define(['angular', 'app'], function (angular, app) {
    'use strict';
    app.controller('SASTAssessmentController', function ($scope, $rootScope, $state, $stateParams, $log, SASTAssessmentService, SASTRecommendationEngine, $timeout) {
        // Controller is used to show all or a subset of the Questions.
        // Functions:
        //      -
        // Flow:
        //      - Load the questions.
        //      - If have choices selected, calculate the score
        //

        $log.info('SASTAssessmentController:');
        // ---------------------------------------------------------------------------------------------------------
        // -    Init Section, setting default values.
        // ---------------------------------------------------------------------------------------------------------
        var service = new SASTAssessmentService(),
            questions = service.returnQuestions();

        $scope.questions = questions;

        $scope.updateScore = function () {
            $log.info($scope.questions);

            var newScore = SASTRecommendationEngine.score($scope.questions);
            $rootScope.$emit('UpdateScore', newScore);
        };
        // The timeout occurs to let the page render then update the score.
        // Otherwise the top score doesn't populate on page load.
        $timeout(function () {
            $scope.updateScore();
        }, 500);

        // ---------------------------------------------------------------------------------------------------------
        // -    Module Function - disableSelectTasks
        // ---------------------------------------------------------------------------------------------------------
        $scope.disableSelectTasks = function () {
            // Here we have to create a copy of hte questions, and remove Question 5 (the checkbox)
            // Can't have a checkbox (with an unchecked value of 0) be required; otherwise the minimum
            // score would always be 1.
            $scope.allRequiredQuestions = angular.copy($scope.questions);
            $scope.allRequiredQuestions.splice(4, 1);
            return !(_.every($scope.allRequiredQuestions, 'choice'));
        };

        // ---------------------------------------------------------------------------------------------------------
        // -    Module Function - next
        // ---------------------------------------------------------------------------------------------------------
        // Ok, save the questions here, using the assessment_service.
        // We use the recommendationEngine to save the algorithms.
        $scope.next = function () {
            service.save($scope.questions);
            $scope.score = SASTRecommendationEngine.score($scope.questions);

            $state.go('^.recommendation', {
                section: $stateParams.section,
                score: $scope.score,
                sectionIndex: $stateParams.sectionIndex
            });
        };
    });
});