/*jslint node: true, unparam: true */
/*global define, describe, beforeEach, inject, it, expect */

define([
    'SASTConstants',
    'SASTRecommendationsController',
    'SASTAssessmentService'
], function (SASTConstants,
             SASTRecommendationsController,
             SASTAssessmentService) {
    'use strict';

    describe("The SASTRecommendationsController controller", function () {
        var controller,
            createController,
            scope,
            stateParams,
            SASTRecommendationEngineMock,
            stateParamsMock;

        // -------------------------------------------------------------------------------------------------------------
        // -   Variables
        // -------------------------------------------------------------------------------------------------------------
        SASTRecommendationEngineMock = {
            score: function () {
                return 4;
            }
        };
        stateParamsMock = {
            section: "algorithm-01",
            sectionIndex: 1,
            sectionName: "Transfer: Bed to Chair, Chair to Toilet, Chair to Chair, Car to Chair",
            score: 0
        };


        // -------------------------------------------------------------------------------------------------------------
        // -   Building out Controller and Inserting $provide and spy's
        // -------------------------------------------------------------------------------------------------------------
        beforeEach(function () {
            module('angularTemplateApp');

            module(function ($provide) {
                $provide.value('SASTRecommendationEngine', SASTRecommendationEngineMock);
            });

            inject(function ($controller, $rootScope, _SASTConstants_, _SASTAssessmentService_) {
                scope = $rootScope.$new();
                stateParams = stateParamsMock;
                SASTConstants = _SASTConstants_;
                SASTAssessmentService = _SASTAssessmentService_;
                scope.score = 4;

                createController = function () {
                    return $controller('SASTRecommendationsController', {
                        $scope: scope,
                        SASTAssessmentService: _SASTAssessmentService_,
                        SASTConstants: SASTConstants,
                        SASTRecommendationEngine: SASTRecommendationEngineMock,
                        $stateParams: stateParamsMock
                    });
                };
                // this way if I need to re-instantiate the controller within and 'it' I could by calling this line.
                controller = createController();
            });
        });

        describe("should load", function () {
            it("the controller", function () {
                expect(scope.score).toBeDefined();
                expect(scope.solution).toEqual(SASTConstants.algorithms[0].solutions[4]);
            });
        });
    });
});
