/*jslint node: true, unparam: true */
/*globals define */

define([
    'app',
    'modalService',
    'SASTConstants',
    'SASTPrimaryWrapperController',
    'SASTAssessmentController',
    'SASTRecommendationsController',
    'SASTAssessmentService',
    'SASTRecommendationEngine',
    'ScoringInfoController',
    'scoringInfoService'
], function (app) {
    "use strict";

    app.config(function ($stateProvider, $urlRouterProvider) {
        $stateProvider
            .state('main.two-panel.secondary-navigation.sast', {
                url: '/sast',
                appNavIndex: 3,
                data: {
                    appletName: "Scoring & Algorithms for Specific Tasks",
                    headerTitle: "Safe Patient Handling",

                    panel: "secondary",
                    baseStateName: 'main.two-panel.secondary-navigation.sast',
                    initialState: "assessment",
                    paramSections: [
                        {
                            paramValue: "algorithm-01",
                            section: "algorithm-01",
                            buttonClass: "option1",
                            sectionName: "Transfer: Bed to Chair, Chair to Toilet, Chair to Chair, Car to Chair",
                            sectionIndex: 1,
                            algorithms: []
                        }, {
                            paramValue: "algorithm-02",
                            section: "algorithm-02",
                            buttonClass: "option2",
                            sectionName: "Lateral Transfer to & From: Bed to Stretcher/Trolley",
                            sectionIndex: 2,
                            algorithms: []
                        }, {
                            paramValue: "algorithm-03",
                            section: "algorithm-03",
                            buttonClass: "option3",
                            sectionName: "Transfer To & From: Chair to Stretcher or Chair to Exam Table",
                            sectionIndex: 3,
                            algorithms: []
                        }, {
                            paramValue: "algorithm-04",
                            section: "algorithm-04",
                            buttonClass: "option4",
                            sectionName: "Reposition up in Bed: Side to Side, Up in Bed",
                            sectionIndex: 4,
                            algorithms: []
                        }, {
                            paramValue: "algorithm-05",
                            section: "algorithm-05",
                            buttonClass: "option5",
                            sectionName: "Reposition in Chair: Wheelchair & Dependency Chair",
                            sectionIndex: 5,
                            algorithms: []
                        }, {
                            paramValue: "algorithm-06",
                            section: "algorithm-06",
                            buttonClass: "option6",
                            sectionName: "Transfer a Patient up from floor",
                            sectionIndex: 6,
                            algorithms: []
                        }
                    ],
                    headerButton: {
                        iconClass: "icon icon-info",
                        text: "Scoring Information",
                        service: "scoringInfoService"
                    }
                },
                views: {
                    'primary@main.two-panel': {
                        templateUrl: 'src/modules/sast/components/primary-wrapper/sast_primary_wrapper_template.html',
                        controller: 'SASTPrimaryWrapperController'
                    }
                }
            })
            // Assessment is the Question List
            .state('main.two-panel.secondary-navigation.sast.assessment', {
                url: '/{section:algorithm-01|algorithm-02|algorithm-03|algorithm-04|algorithm-05|algorithm-06}/{sectionIndex}',
                data: {
                    panel: "primary",
                    backState: "^"
                },
                views: {
                    'primary-content': {
                        templateUrl: 'src/modules/sast/pages/assessment/sast_assessment_template.html',
                        controller: 'SASTAssessmentController'
                    }
                },
                params: {
                    algorithms: [],
                    section: "algorithm-01",
                    sectionName: "Transfer: Bed to Chair, Chair to Toilet, Chair to Chair, Car to Chair"
                }
            })
            .state('main.two-panel.secondary-navigation.sast.recommendation', {
                url: '/{section:algorithm-01|algorithm-02|algorithm-03|algorithm-04|algorithm-05|algorithm-06}/{sectionIndex}',
                data: {
                    panel: "primary",
                    backState: "^"
                },
                views: {
                    'primary-content': {
                        templateUrl: 'src/modules/sast/pages/recommendations/sast_recommendations_template.html',
                        controller: 'SASTRecommendationsController'
                    }
                },
                params: {
                    algorithms: [],
                    section: "algorithm-01",
                    sectionName: "Transfer: Bed to Chair, Chair to Toilet, Chair to Chair, Car to Chair",
                    score: 0
                }
            });
    });
});
