/*jslint node: true */
/*global define */

define([
    'app',
    'eula_service',
    'modalService',
    'EulaController',
    'HeaderController',
    'FooterController',
    'MainController',
    'HelpController',
    'AboutController',
    'accessibleDatePickerServices',
    'DateControls',
    'SecondaryNavigationController',
    'HomeModule',
    'CPAAModule',
    'AORNModule',
    'NAONModule',
    'SASTModule',
    'ResourcesModule'
], function (app) {
    "use strict";

    app.config(function ($stateProvider, unsavedWarningsConfigProvider) {

        $stateProvider
            .state('main', {
                abstract: true,
                data: {
                    appletName: "Safe Patient Handling"
                },
                views: {
                    "": {
                        templateUrl: 'src/container/main/main_template.html',
                        controller: 'MainController'
                    },
                    'header@main': {
                        templateUrl: 'src/container/components/header/header_template.html',
                        controller: 'HeaderController'
                    },
                    'footer@main': {
                        templateUrl: 'src/container/components/footer/footer_template.html',
                        controller: 'FooterController'
                    }
                },
                resolve: {
                    eula: function ($q, eulaService) {
                        var defer = $q.defer();
                        eulaService.isAccepted() ? defer.resolve() : defer.reject({error: 'eula'});
                        return defer.promise;
                    }
                }
            })
            .state('main.two-panel', {
                abstract: true,
                templateUrl: 'src/container/components/content/two-column_template.html',
                controller: 'TwoPanelController'
            })
            .state('main.two-panel.secondary-navigation', {
                abstract: true,
                views: {
                    'secondary@main.two-panel': {
                        templateUrl: "src/container/components/content/secondary-navigation/secondary-navigation_template.html",
                        controller: "SecondaryNavigationController"
                    }
                }
            })
            .state('eula', {
                url: '/eula',
                templateUrl: 'src/core/eula/eula_template.html',
                controller: 'EulaController'
            })
            .state('nonStateRedirect', {
                url: '/redirecting'
            });

        unsavedWarningsConfigProvider.useTranslateService = false;
    });

    app.run(function ($rootScope, $state, $window, pageService) {

        $rootScope.$on('$stateChangeError', function (event, toState, toParams, fromState, fromParams, error) {

            if (error.redirect) {
                $state.go('nonStateRedirect');
                $window.open(error.redirectUrl, '_self', null, true);
            } else {
                switch (error.error) {
                    case 'eula':
                        $state.go('eula');
                        break;
                }
            }

        });
    });
});